'translated this 08.08.10@11:09PM by mjmendozaIV
#Include Once "kageExterns.bi"

''
' Parser for KonsolScript's <b>Game</b> class.
'
' This enables KonsolScript to do tile-based game rendering.<br>
' TODO:
' <ul>
'  <li>AttachMap - Adds a map to be used for the game. (is this not the same with Game:Render)
'  <li>Load - loads a game state.
'  <li>Save - saves a game state.
'  <li>Start - runs the game. (Game:Render does the same)
' </ul>

''
' Used by friendly debugging
'
' @return String
' @remarks None.
Dim Shared cGameTxt As String

''
' Parses and interprets KonsolScript's <b>Game</b> functions.
'
' @param	iIndex - Index of the parsed script
' @return   New index value of the parsed script.
' @remarks Needs more research and refining.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' No need to make game buffers <i>(There should only be one instance of a game per session).</i><br>
' Game class currently contains the functions:
' <ul>
' <li><b><a href="kse_Game.GameAttachEntity.html">AttachEntity</a></b> - Adds an entity to the game.
' <li><b><a href="kse_Game.GameEntitySetX.html">EntitySetX</a></b> - Sets an entity's X location.
' <li><b><a href="kse_Game.GameEntitySetY.html">EntitySetY</a></b> - Sets an entity's Y location.
' <li><b><a href="kse_Game.GameEntityGetX.html">EntityGetX</a></b> - Returns an entity's X location.
' <li><b><a href="kse_Game.GameEntityGetY.html">EntityGetY</a></b> - Returns an entity's Y location.
' <li><b><a href="kse_Game.GameFocusEntity.html">FocusEntity</a></b> - Gives focus on an entity.
' <li><b><a href="kse_Game.GameGetCameraX.html">GetCameraX</a></b> - Returns the game camera's X location.
' <li><b><a href="kse_Game.GameGetCameraY.html">GetCameraY</a></b> - Returns the game camera's Y location.
' <li><b><a href="kse_Game.GameSetCameraX.html">SetCameraX</a></b> - Sets the game camera's X location.
' <li><b><a href="kse_Game.GameSetCameraY.html">SetCameraY</a></b> - Sets the game camera's Y location.
' <li><b><a href="kse_Game.GameSetPlayer.html">SetPlayer</a></b> - Set which entity is to be user-controlled.
' <li><b><a href="kse_Game.GamePause.html">Pause</a></b> - Pauses a game from rendering.
' <li><b><a href="kse_Game.GameResume.html">Resume</a></b> - Resumes a game from pausing.
' <li><b><a href="kse_Game.GameStop.html">Stop</a></b> - Stops the game.
' <!--  TO DO -->
' <!-- li><b><a href="kse_Game.GameAttachMap.html">AttachMap</a></b> - Adds a map to be used for the game. -->
' <!-- li><b><a href="kse_Game.GameLoad.html">Load</a></b> - Loads a game state. -->
' <!-- li><b><a href="kse_Game.GameSave.html">Save</a></b> - Saves a game state. -->
' <!-- li><b><a href="kse_Game.GameStart.html">Start</a></b> - Runs the game. -->
' </ul>
'</div>
Public Function SynGame(ByRef iIndex As Long) As Integer
	iIndex = GetNearestTokenIndex(kseCOLON, "Game", iIndex)   'look for		:
	iIndex = GetNextIndex(iIndex)
	Select Case kseToken(iIndex).sToken
		'Case "Play"
		'	Return GameLoad(iIndex)
		Case "Render"
			Return GameRender(iIndex)
		Case "SetCameraX"
			Return GameSetCameraX(iIndex)
		Case "SetCameraY"
			Return GameSetCameraY(iIndex)
		Case "GetCameraX"
			Return GameGetCameraX(iIndex)
		Case "GetCameraY"
			Return GameGetCameraY(iIndex)
		'Case "Degenerate"
		'	Return GameDegenerate(iIndex)
		Case "Resume"
			Return GameResume(iIndex)
		Case "Pause"
			Return GamePause(iIndex)
		Case "Stop"
			Return GameStop(iIndex)
		Case "EntitySetX"
			Return GameEntitySetX(iIndex)
		Case "EntitySetY"
			Return GameEntitySetY(iIndex)
		Case "EntityGetX"
			Return GameEntityGetX(iIndex)
		Case "EntityGetY"
			Return GameEntityGetY(iIndex)
		Case "AttachEntity"
			Return GameAttachEntity(iIndex)
		'Case "DetachEntity"
		'	Return GameDetachEntity(iIndex)
		Case "SetPlayer"
			Return GameSetPlayer(iIndex)
		Case "FocusEntity"
			Return GameFocusEntity(iIndex)
		Case Else
			Engine_Message KSE_GetErrFileSrc(iIndex) & "(  " & KSE_GetToken(iIndex) & "  ) is not in Game Class.", "ERROR AT LINE " & kseToken(iIndex).iLine, 1
	End Select
	'to be implemented once KonsolScript is ready for production release
	'Return GetNearestTokenIndex(kseSEMICOLON, kseCLOSE_PARENTHESIS, SynMap)   'look for		;
End Function

''
' <b>Game:SetCameraX</b> - Sets the game camera's X location.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Sets the X location of the camera of the current map being rendered. Can be used for customized cut-scene.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>Number <b>value</b> - desired new X location.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  Game:SetCameraX(100)
' </pre>
'</div>
Private Function GameSetCameraX(ByRef iIndex As Long) As Integer
	Dim iVal As Integer
	
	iIndex = Game_SkipOpenParenthesisAfter("SetCameraX", iIndex)
		
		iVal = KSE_GetTokenNumValueOf(iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("value", iIndex)
	'EXECUTE SCRIPT =====================================================================
	kseMap(kseMAP_CURRENT_RENDER).iCamX = iVal
	kseMap(kseMAP_CURRENT_RENDER).iX = -kseMap(kseMAP_CURRENT_RENDER).iCamX
	
	Return iIndex
End Function

''
' <b>Game:SetCameraY</b> - Sets the game camera's Y location.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Sets the Y location of the camera of the current map being rendered. Can be used for customized cut-scene.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>Number <b>value</b> - desired new Y location.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  Game:SetCameraY(100)
' </pre>
'</div>
Private Function GameSetCameraY(ByRef iIndex As Long) As Integer
	Dim iVal As Integer
	
	iIndex = Game_SkipOpenParenthesisAfter("SetCameraY", iIndex)
		
		iVal = KSE_GetTokenNumValueOf(iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("value", iIndex)
	'EXECUTE SCRIPT =====================================================================
	kseMap(kseMAP_CURRENT_RENDER).iCamY = iVal
	kseMap(kseMAP_CURRENT_RENDER).iY = -kseMap(kseMAP_CURRENT_RENDER).iCamY
	
	Return iIndex
End Function

''
' <b>Game:GetCameraX</b> - Returns the game camera's X location.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Returns the X location of the camera. Can be used for customized cut-scene animation.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>Number <b>return</b> - a <b>Number</b> variable for return value.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  Var:Number xLoc;
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  Game:GetCameraX(xLoc)
' </pre>
'</div>
Private Function GameGetCameraX(ByRef iIndex As Long) As Integer
	Dim iIdx As Integer
	
	iIndex = Game_SkipOpenParenthesisAfter("GetCameraX", iIndex)
		
		iIndex = GetNextIndex(iIndex)
		iIdx = iIndex
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex)
	'EXECUTE SCRIPT =====================================================================
	SetNumValue iIdx, CDbl(kseMap(kseMAP_CURRENT_RENDER).iCamX)
	
	Return iIndex
End Function

''
' <b>Game:GetCameraY</b> - Returns the game camera's Y location.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Returns the Y location of the camera. Can be used for customized cut-scene animation.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>Number <b>return</b> - a <b>Number</b> variable for return value.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  Var:Number yLoc;
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  Game:GetCameraY(yLoc)
' </pre>
'</div>
Private Function GameGetCameraY(ByRef iIndex As Long) As Integer
	Dim iIdx As Integer
	
	iIndex = Game_SkipOpenParenthesisAfter("GetCameraY", iIndex)
		
		iIndex = GetNextIndex(iIndex)
		iIdx = iIndex
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex)
	'EXECUTE SCRIPT =====================================================================
	SetNumValue iIdx, CDbl(kseMap(kseMAP_CURRENT_RENDER).iCamY)
	
	Return iIndex
End Function

Private Function GameSetPlayer(ByRef iIndex As Long) As Integer
	Dim sEntity As String
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	iIndex = Game_SkipOpenParenthesisAfter("SetPlayer", iIndex)
		
		sEntity = KSE_GetTokenStrValueOf(iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex)
	'EXECUTE SCRIPT =====================================================================
	Dim bGoCheck As Byte
	If kseMap(iMID).iMoveMode = kseMAP_MOVE_TILEBASED_MODE Then
		If iMapMoveCtr >= iMapMoveCtrMax Then
			'make sure to finish player's tile-based movement
			bGoCheck = 1
		End If
	Else
		bGoCheck = 1
	End If
	
	If bGoCheck = 1 Then
		'change current player when player-entity is not moving
		For i As Integer = 1 To kseMap(iMID).EntitiesCtr - 1
			If LCase(kseMap(iMID).Entities[i].sName) = LCase(sEntity) Then
				If kseMap(iMID).iPlayer <> i Then
					'NOTE: this condition should be kept separated with the first 2 condition
					If (kseMap(iMID).iPlayer <> 0 And kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 1) _
							Or kseMap(iMID).iPlayer = 0 Then
						'change current player when player-entity is not moving
						kseMap(iMID).iPlayer = i
					End If
				End If
				Exit For
			End If
		Next
	End If
	
	
	Return iIndex

End Function

''
' <b>Game:EntitySetX</b> - Relocates entity's <b>x</b> location.
'
' @param	iIndex - Index of the parsed script
' @return   New index value of the parsed script.
' @remarks None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Relocates entity's <b>x</b> location.<br>
' Please note, that the value should have no decimal value -- assigning decimals will be rounded.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>Number <b>value</b> - desired new <b>x</b> value.
'  <li>String <b>entity_name</b> - name of the entity you want to move.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  //provided myEntity is a defined entity
'  Game:AttachEntity(myEntity, "creek") //attach the entity
'  Game:EntitySetX(100, "creek")		//set new x location
' </pre>
'</div>
Private Function GameEntitySetX(ByRef iIndex As Long) As Integer
	Dim sEntity As String
	Dim iMID As Integer
	Dim value As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	iIndex = Game_SkipOpenParenthesisAfter("EntitySetX", iIndex)
	
		value = KSE_GetTokenNumValueOf(iIndex)
	
	iIndex = Game_SkipCommaAfter("value", iIndex)
	
		sEntity = KSE_GetTokenStrValueOf(iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex)
	'EXECUTE SCRIPT ====================================================
	Dim bGoCheck As Byte
	If kseMap(iMID).iMoveMode = kseMAP_MOVE_TILEBASED_MODE Then
		'make sure to finish player's tile-based movement
		If iMapMoveCtr >= iMapMoveCtrMax Then
			bGoCheck = 1
		End If
	Else
		bGoCheck = 1
	End If
	
	If bGoCheck = 1 Then
		'change current player when player-entity is not moving
		For i As Integer = 1 To kseMap(iMID).EntitiesCtr - 1
			If LCase(kseMap(iMID).Entities[i].sName) = LCase(sEntity) Then
				'NOTE: this condition should be kept separated with the first 2 condition
				If (kseMap(iMID).iPlayer <> 0 And kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 1) _
						Or kseMap(iMID).iPlayer = 0 Then
					Dim As Integer tile_center_x = (kseMap(iMID).MapDimension / 2) - (kseMap(iMID).Entities[i].iWidth / 2)
					Dim As Integer value_excess = value Mod kseMap(iMID).MapDimension
					
					value = value - value_excess
					
					If value_excess > tile_center_x Then
						value = value + tile_center_x
					End If
					
					kseMap(iMID).Entities[i].iX = Int(value)
					
					Exit For
				End If
			End If
		Next
	End If
	
	Return iIndex
End Function

''
' <b>Game:EntitySetY</b> - Relocates entity's <b>y</b> location.
'
' @param	iIndex - Index of the parsed script
' @return   New index value of the parsed script.
' @remarks None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Relocates entity's <b>y</b> location.<br>
' Please note, that the value should have no decimal value -- assigning decimals will be rounded.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>Number <b>value</b> - desired new <b>y</b> value.
'  <li>String <b>entity_name</b> - name of the entity you want to move.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  //provided myEntity is a defined entity
'  Game:AttachEntity(myEntity, "creek") //attach the entity
'  Game:EntitySetY(100, "creek")		//set new y location
' </pre>
'</div>
Private Function GameEntitySetY(ByRef iIndex As Long) As Integer
	Dim sEntity As String
	Dim iMID As Integer
	Dim value As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	iIndex = Game_SkipOpenParenthesisAfter("EntitySetY", iIndex)
		
		value = KSE_GetTokenNumValueOf(iIndex)
	
	iIndex = Game_SkipCommaAfter("value", iIndex)
	
		sEntity = KSE_GetTokenStrValueOf(iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex)
	'EXECUTE SCRIPT =====================================================================
	Dim bGoCheck As Byte
	If kseMap(iMID).iMoveMode = kseMAP_MOVE_TILEBASED_MODE Then
		'make sure to finish player's tile-based movement
		If iMapMoveCtr >= iMapMoveCtrMax Then
			bGoCheck = 1
		End If
	Else
		bGoCheck = 1
	End If
	
	If bGoCheck = 1 Then
		'change current player when player-entity is not moving
		For i As Integer = 1 To kseMap(iMID).EntitiesCtr - 1
			If LCase(kseMap(iMID).Entities[i].sName) = LCase(sEntity) Then
				'NOTE: this condition should be kept separated with the first 2 condition
				If (kseMap(iMID).iPlayer <> 0 And kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 1) _
						Or kseMap(iMID).iPlayer = 0 Then
					Dim As Integer tile_center_y = (kseMap(iMID).MapDimension / 2) - (kseMap(iMID).Entities[i].iHeight / 2)
					Dim As Integer value_excess = value Mod kseMap(iMID).MapDimension
					
					value = value - value_excess
					
					If value_excess > tile_center_y Then
						value = value + tile_center_y
					End If
					
					kseMap(iMID).Entities[i].iY = Int(value)
					'kseMap(iMID).Entities[i].iY = (Int(value / kseMap(iMID).MapDimension) * kseMap(iMID).MapDimension) - kseMap(iMID).Entities[i].iHeight
					Exit For
				End If
			End If
		Next
	End If
	
	Return iIndex
End Function

''
' <b>Game:EntityGetX</b> - Returns the X location of an entity.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Returns the Y location of an entity.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>String <b>entity</b> - instance name of the entity to be attached.
'  <li>Number <b>return</b> - Number variable to place the value.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  Var:Number myYloc;
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  //provided that myEntity is an instance of a defined entity (See Game:AttachEntity)
'  Game:EntityGetY("myEntity", myYloc)
' </pre>
'</div>
Private Function GameEntityGetX(ByRef iIndex As Long) As Integer
	Dim sEntity As String
	Dim iRet As Integer
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	iIndex = Game_SkipOpenParenthesisAfter("EntityGetX", iIndex)
		
		sEntity = KSE_GetTokenStrValueOf(iIndex)
	
	iIndex = Game_SkipCommaAfter("entity", iIndex)
	
		iIndex = GetNextIndex(iIndex)
		iRet = iIndex
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex)
	'EXECUTE SCRIPT =====================================================================
	For i As Integer = 1 To kseMap(iMID).EntitiesCtr - 1
		If LCase(kseMap(iMID).Entities[i].sName) = LCase(sEntity) Then
			SetNumValue iRet, CDbl(kseMap(iMID).Entities[i].iX)
			Exit For
		End If
	Next
	
	Return iIndex
End Function

''
' <b>Game:EntityGetY</b> - Returns the Y location of an entity.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Returns the Y location of an entity.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>String <b>entity</b> - instance name of the entity to be attached.
'  <li>Number <b>return</b> - Number variable to place the value.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  Var:Number myYloc;
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  //provided that myEntity is an instance of a defined entity (See Game:AttachEntity)
'  Game:EntityGetY("myEntity", myYloc)
' </pre>
'</div>
Private Function GameEntityGetY(ByRef iIndex As Long) As Integer
	Dim sEntity As String
	Dim iRet As Integer
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	iIndex = Game_SkipOpenParenthesisAfter("EntityGetY", iIndex)
		
		sEntity = KSE_GetTokenStrValueOf(iIndex)
	
	iIndex = Game_SkipCommaAfter("entity", iIndex)
	
		iIndex = GetNextIndex(iIndex)
		iRet = iIndex
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex)
	'EXECUTE SCRIPT =====================================================================
	For i As Integer = 1 To kseMap(iMID).EntitiesCtr - 1
		If LCase(kseMap(iMID).Entities[i].sName) = LCase(sEntity) Then
			SetNumValue iRet, CDbl(kseMap(iMID).Entities[i].iY)
			Exit For
		End If
	Next
	
	Return iIndex
End Function

Public Sub Engine_RunScript(sScriptFile As String, fxName As String, sFileExt As String = ".ks", bWarn As Byte = 1)
	Dim i As Integer
	For i = 0 To iFunctionCtr
		If kseFunction(i).fName = fxName _
				And (UCase(colSourceName.ItemAt(kseToken(kseFunction(i).fStartIndex).iSrc)) = UCase(sScriptFile & sFileExt) _
				Or InStr(1, UCase(colSourceName.ItemAt(kseToken(kseFunction(i).fStartIndex).iSrc)), UCase(kseSLASH & sScriptFile & sFileExt)) > 0) Then
			kseFunction(i).RunScript
			Exit For
		End If
	Next
	
	If bWarn Then
		If i > iFunctionCtr Then Engine_Log "Warning: (  function " & fxName & "  ) of (  " & sScriptFile & "  ) does not exist."
	End If
End Sub

Private Function ReDim_MapEntity(new_list_count As Integer, map_idx As Integer) As Byte
	If map_idx < 0 Then Return 0 'failed
	If map_idx > iMapCtr Then Return 0 'failed
	Dim max_count As Integer
	Dim i As Integer
	Dim As Integer old_list_count = kseMap(map_idx).EntitiesCtr
	
	'transfer the data to temp
	Dim As typeEntity Ptr tmpEnt = CAllocate(old_list_count*SizeOf(typeEntity))
		For i = 0 To old_list_count - 1
			if i > 0 Then '0 is for tmp
				tmpEnt[i] = kseMap(map_idx).Entities[i]
			End If
		Next
	DeAllocate kseMap(map_idx).Entities
	'get max list count
		max_count = IIf(new_list_count > old_list_count, new_list_count, old_list_count)
		'kseMap(map_idx).EntitiesCtr = max_count
	
	'allocate new size
		kseMap(map_idx).Entities = CAllocate((max_count+1)*SizeOf(typeEntity)) 'always add extra for tmp
		If kseMap(map_idx).Entities = 0 Then Return 0
	'put back data to source
	For i = 0 To max_count - 1 ' last index has no previous value
		If i <= old_list_count - 1 And i > 0 Then
			kseMap(map_idx).Entities[i] = tmpEnt[i]
		End If
	Next
	Return 1
End Function

Private Function ReDim_MapEntityName(new_list_count As Integer,map_idx As Integer) As Byte
	If map_idx < 0 Then Return 0 'failed
	If map_idx > iMapCtr Then Return 0 'failed
	Dim max_count As Integer
	Dim i As Integer
	Dim As Integer old_list_count = kseMap(map_idx).EntitiesCtr
	
	'transfer the data to temp
	Dim As String tmpEnt(old_list_count)
		For i = 0 To old_list_count - 1
			If i > 0 Then '0 is for tmp
				tmpEnt(i) = kseMap(map_idx).EntitiesName[i]
			End If
		Next
	DeAllocate kseMap(map_idx).EntitiesName
	'get max list count
	max_count = IIf(new_list_count > old_list_count, new_list_count, old_list_count)
	
	'allocate new size
		kseMap(map_idx).EntitiesName = Callocate((max_count+1)*SizeOf(String)) 'always add extra for tmp
		
		If kseMap(map_idx).EntitiesName = 0 Then Return 0
	
	'put back data to source
	For i = 0 To max_count - 1 ' start i to 1 since 0 is for tmp purposes
		If i <= old_list_count - 1 And i > 0 Then
			kseMap(map_idx).EntitiesName[i] = tmpEnt(i)
		End If
	Next
	Return 1
End Function

''
' <b>Game:AttachEntity</b> - Adds an entity to the game.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Creates an instance from a defined entity and attaches it to the game.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>String <b>entity</b> - instance name of the entity to be attached.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  //provided that myEntity is an instance of a defined entity (See Entity:New)
'  Game:AttachEntity(myEntity, "myEntityInstance")
'  //from then on, accessing the attached entity will be by its name, myEntityInstance
' </pre>
'</div>
Private Function GameAttachEntity(ByRef iIndex As Long) As Integer
	Dim iEntity As Integer
	Dim sName As String
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	iIndex = Game_SkipOpenParenthesisAfter("AttachEntity", iIndex)
		
		iEntity = KSE_GetTokenNumValueOf(iIndex)
		
	iIndex = Game_SkipCommaAfter("entity", iIndex)
	
		sName = KSE_GetTokenStrValueOf(iIndex)
		
	iIndex = Game_SkipCloseParenthesisAfter("name", iIndex)
	'EXECUTE SCRIPT ====================================================
	'Redim Preserve kseMap(iMID).Entities(kseMap(iMID).EntitiesCtr) As typeEntity

	'NOTE: these statements are meant to be separated into 2 ifs
	If iEntity < 1 Then
		Engine_Message KSE_GetErrFileSrc(iIndex) & "Make sure entity file was loaded.", "LOGIC ERROR AT LINE " & kseToken(iIndex).iLine, 1
	End If
	
	If iMID < 1 Or iMID > iMapCtr Then
	   Engine_ERR_NO_MAP iIndex
	ElseIf kseMap(iMID).bUsed = 0 Then
	   Engine_ERR_NO_MAP iIndex
	End If
	If ReDim_MapEntity(kseMap(iMID).EntitiesCtr+1, iMID) = 0 Then Engine_ERR_NO_MAP iIndex
	
	With kseMap(iMID).Entities[kseMap(iMID).EntitiesCtr]
		.bHide = kseEntity(iEntity).bHide
		.bStatic = 1
		.bUsed = 1

		.iAnmt_LeftTileMax = kseEntity(iEntity).iAnmt_LeftTileMax
			fbUDTtoUDT kseEntity(iEntity).iAnmt_LeftTiles, .iAnmt_LeftTiles,.iAnmt_LeftTileMax
'		.iAnmt_LeftCtr = kseEntity(iEntity).iAnmt_LeftCtr
'			.iAnmt_LeftTiles() = kseEntity(iEntity).iAnmt_LeftTiles()
		
		.iAnmt_RightTileMax = kseEntity(iEntity).iAnmt_RightTileMax
			fbUDTtoUDT kseEntity(iEntity).iAnmt_RightTiles, .iAnmt_RightTiles,.iAnmt_RightTileMax
'		.iAnmt_RightCtr = kseEntity(iEntity).iAnmt_RightCtr
'			.iAnmt_RightTiles() = kseEntity(iEntity).iAnmt_RightTiles()
		
		.iAnmt_DownTileMax = kseEntity(iEntity).iAnmt_DownTileMax
		   fbUDTtoUDT kseEntity(iEntity).iAnmt_DownTiles, .iAnmt_DownTiles,.iAnmt_DownTileMax
'		.iAnmt_DownCtr = kseEntity(iEntity).iAnmt_DownCtr
'			.iAnmt_DownTiles() = kseEntity(iEntity).iAnmt_DownTiles()
		
		.iAnmt_UpTileMax = kseEntity(iEntity).iAnmt_UpTileMax
			fbUDTtoUDT kseEntity(iEntity).iAnmt_UpTiles, .iAnmt_UpTiles,.iAnmt_UpTileMax
'		.iAnmt_UpCtr = kseEntity(iEntity).iAnmt_UpCtr
'			.iAnmt_UpTiles() = kseEntity(iEntity).iAnmt_UpTiles()

		.iDirection = kseEntity(iEntity).iDirection
		.iHeight = kseEntity(iEntity).iHeight
		.IsAttachedToMap = 1
		.iSpriteHandlerMax  = kseEntity(iEntity).iSpriteHandlerMax
			fbUDTtoUDT kseEntity(iEntity).iSpriteHandlers, .iSpriteHandlers, .iSpriteHandlerMax
'		.iSpriteHandlers() = kseEntity(iEntity).iSpriteHandlers()
		.iSpriteIndex = kseEntity(iEntity).iSpriteIndex
		.iWidth = kseEntity(iEntity).iWidth
		.iX = kseEntity(iEntity).iX
		.iY = kseEntity(iEntity).iY
		.sName = sName
	End With
	
'	ReDim Preserve kseMap(iMID).EntitiesName(kseMap(iMID).EntitiesCtr) As String
	If ReDim_MapEntityName(kseMap(iMID).EntitiesCtr+1, iMID) = 1 Then
		kseMap(iMID).EntitiesName[kseMap(iMID).EntitiesCtr] = sName
	End If
	
	kseMap(iMID).EntitiesCtr = kseMap(iMID).EntitiesCtr + 1
	
	'If kseEntity(iEntity).IsAttachedToMap Then Engine_RelocateEntityOnMap iEntity
	'kseEntity(iEntity).IsAttachedToMap = 1
	'Engine_LocateEntityOnMap iEntity, iMid
	
	Return iIndex
End Function

'TODO: NOT DONE YET
Private Function GameDetachEntity(ByRef iIndex As Long) As Integer
	Dim sEntity As String
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	iIndex = Game_SkipOpenParenthesisAfter("DetachEntity", iIndex)
	
		sEntity = KSE_GetTokenStrValueOf(iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex)
	'EXECUTE SCRIPT ====================================================
'	ReDim Preserve iMapEntityHandlers(iMapEntityHandlersCtr) As Integer
'	iMapEntityHandlers(iMapEntityHandlersCtr) = sEntity
'	iMapEntityHandlersCtr = iMapEntityHandlersCtr + 1
	For i As Integer = 0 To 10 'TODO: Edit me
		With kseMap(iMID).Entities[kseMap(iMID).EntitiesCtr]
			'.bHide = kseEntity(i).bHide
			If .sName = sEntity Then
				.bUsed = 0
				'kseEntity(iEntity).IsAttachedToMap = 0
			End If
		End With
	Next
		
	Return iIndex
End Function

Private Function GamePause(ByRef iIndex As Long) As Integer
	iIndex = Game_SkipOpenParenthesisAfter("Pause", iIndex)
	
	'	iMID = KSE_GetTokenNumValueOf(iIndex)
		
	iIndex = Game_SkipCloseParenthesisAfter("(", iIndex)
	'EXECUTE SCRIPT ====================================================
	'iMID = kseMAP_CURRENT_RENDER
	kseMap(kseMAP_CURRENT_RENDER).bRendering = 0
	bMapRendering = 0
	
	Return iIndex
End Function

Private Function GameResume(ByRef iIndex As Long) As Integer
	iIndex = Game_SkipOpenParenthesisAfter("Resume", iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("(", iIndex)
	'EXECUTE SCRIPT ====================================================
	kseMap(kseMAP_CURRENT_RENDER).bRendering = 1
	bMapRendering = 1
	
	Return iIndex
End Function

Private Function GameStop(ByRef iIndex As Long) As Integer
	iIndex = Game_SkipOpenParenthesisAfter("Stop", iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("(", iIndex)
	'EXECUTE SCRIPT ====================================================
	'iMID = kseMAP_CURRENT_RENDER
	kseMap(kseMAP_CURRENT_RENDER).bRendering = 0
	bMapRendering = 0
	
	kseMap(kseMAP_CURRENT_RENDER).bRender = 0
	bMapRender = 0
	
	Return iIndex
End Function

''
' <b>Game:Render</b> - Runs the game based on the map to be assigned.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Runs the game based on the assigned map.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>Number <b>map</b> - the ID of a generated, defined map.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
' </pre>
'</div>
Private Function GameRender(ByRef iIndex As Long) As Integer
	Dim iLyr As Integer
	Dim iWid As Integer
	Dim iHgt As Integer
	Dim iX As Integer
	Dim iY As Integer
	Dim iMID As Integer
	
	iIndex = Game_SkipOpenParenthesisAfter("Render", iIndex)
	
		iMID = KSE_GetTokenNumValueOf(iIndex)
	
	iIndex = Game_SkipCloseParenthesisAfter("map", iIndex)
	'EXECUTE SCRIPT =====================================================================
	kseMAP_CURRENT_RENDER = iMID 'register current map
	
	Engine_NormalizeEntityLocation
	'hook Map to engine
		kseMap(iMID).bRender = 1
		kseMap(iMID).bRendering = 1
	'hook control on engine
		bMapRender = 1
		bMapRendering = 1
	
	Dim iCtr As Integer
	Dim i As Integer
	Dim iIdx As Integer
	Dim iIdx2 As Integer
	Dim iTile As Integer
	Dim iLyrPicHdlr As Integer
	
	Engine_RunMapScript sMapName & "_onLoad", 0
	Do
		Do
			iCtr = 0
			Engine_CLS()
			For iLyr = 0 To kseMap(iMID).iLayers - 1
				'skip tile rendering if entity-layer
				If iLyr = kseMap(iMID).iLayerObstruct - 1  Then
					iObsLayer = kseMap(iMID).iLayerObstruct 'for use in Engine_IsTileObstruct
					'processing of obstruction data is when entity moves
					'---------------------------------------------------
					'resume counting
					'just add number of entire tiles in a layer
					iCtr = iCtr + kseMap(iMID).iHeight * kseMap(iMID).iWidth
				ElseIf iLyr = kseMap(iMID).iLayerTrigger - 1 Then
					iTrgLayer = kseMap(iMID).iLayerTrigger 'for use in Engine_IsTileObstruct
					'processing of trigger data is when entity moves
					'---------------------------------------------------
					'resume counting
					'just add number of entire tiles in a layer
					iCtr = iCtr + kseMap(iMID).iHeight * kseMap(iMID).iWidth
				'skip tile rendering if entity-layer
				ElseIf iLyr = kseMap(iMID).iLayerEntity - 1 Then
					'render entity instead
					'start with 1; 0 is for 'tmp' purposes
					For i = 1 To kseMap(iMID).EntitiesCtr - 1
						With kseMap(iMID).Entities[i]
							If .bUsed = 1 And .bHide = 0 And .IsAttachedToMap = 1 Then
								'get description
								iIdx2 = KSE_GetSpriteAnimation(CInt(i))
								'--> from old GameRender iIdx = KSE_GetSpriteAnimation(CInt(i))
								'blit entity to Game's screen
								Engine_BlitToRegion iIdx2, _
												 CInt(.iX + Abs(kseMap(iMID).MapDimension / 2) - (.iWidth / 2)), _
												 CInt(.iY + kseMap(iMID).MapDimension - .iHeight), _
												 CInt(kseImg(iIdx).imgWidth), _
												 CInt(kseImg(iIdx).imgHeight), _
												 1, _
												 kseMap(iMID).Map, _
												 CInt(kseImg(kseMap(iMID).Map).imgWidth), _
												 CInt(kseImg(kseMap(iMID).Map).imgHeight)
							End If
						End With
					Next
					'entity layer does not have anything to do with the tiles counting...
					'just add number of entire tiles in a layer
					iCtr = iCtr + kseMap(iMID).iHeight * kseMap(iMID).iWidth
					'===============================================
				Else 'this is the ordinary/normal tile layer used for BG/FG
					For iHgt = 1 To kseMap(iMID).iHeight
						For iWid = 1 To kseMap(iMID).iWidth
							'get the tile index
							iTile = kseMap(iMID).iTileData[iCtr].iIdx
							If iTile > 0 Then 'check if not null
								'locate x and y
								iX = (iWid - 1) * kseMap(iMID).MapDimension
								iY = (iHgt - 1) * kseMap(iMID).MapDimension
								'#doing this first before checking for out-of-bounds will
								'  make sure that animated tile is in sync
								
								'get the picture index of an animated tile only; static tiles where pre-drawn on MapGenerate
								If kseMap(iMID).iTileData[iCtr].bStatic = 0 Then
									iIdx = KSE_GetTileAnimation(CInt(iCtr), iMID)
									iIdx = kseMap(iMID).iTileImgHdlr[iIdx]
									'draw picture to layer
									iLyrPicHdlr = kseMap(iMID).iLayer[iLyr]
									Engine_BlitToRegion iIdx, _
													 CInt(iX), _
													 CInt(iY), _
													 CInt(kseImg(iIdx).imgWidth), _
													 CInt(kseImg(iIdx).imgHeight), _
													 kseMap(iMID).iTileData[iCtr].bTBlit, _
													 iLyrPicHdlr, _
													 CInt(kseImg(iLyrPicHdlr).imgWidth), _
													 CInt(kseImg(iLyrPicHdlr).imgHeight)
								End If
							End If

							'do the counting; start counting from 0
							iCtr = iCtr + 1
							If iCtr > (kseMap(iMID).iLayers * kseMap(iMID).iHeight * kseMap(iMID).iWidth) Then iCtr = 0
						Next
					Next
					'draw the whole layer (without the animating tiles)
					iLyrPicHdlr = kseMap(iMID).iLayer[iLyr]
					'draw layer to map
					Engine_BlitToRegion iLyrPicHdlr, _
									 0, _
									 0, _
									 CInt(kseImg(iLyrPicHdlr).imgWidth), _
									 CInt(kseImg(iLyrPicHdlr).imgHeight), _
									 1, _
									 kseMap(iMID).Map, _
									 CInt(kseImg(kseMap(iMID).Map).imgWidth), _
									 CInt(kseImg(kseMap(iMID).Map).imgHeight)
				End If
			Next
			
			If kseMap(iMID).EntityFocus <> 0 Then
				'remember to follow the entity being focused
				Dim As Integer iEntFocus = kseMap(iMID).EntityFocus
				kseMap(iMID).iX = Int((-kseMap(iMID).Entities[iEntFocus].iX) + (kseImg(quixieSCREEN).imgWidth / 2) - (kseMap(iMID).Entities[iEntFocus].iWidth / 2))
				kseMap(iMID).iY = Int((-kseMap(iMID).Entities[iEntFocus].iY) + (kseImg(quixieSCREEN).imgHeight / 2) - (kseMap(iMID).Entities[iEntFocus].iHeight / 2))
			End If
			
			'draw map to screen
			Engine_BlitToRegion kseMap(iMID).Map, _
							 CInt(kseMap(iMID).iX) - 1, _
							 CInt(kseMap(iMID).iY) - 1, _
							 CInt(kseImg(kseMap(iMID).Map).imgWidth), _
							 CInt(kseImg(kseMap(iMID).Map).imgHeight), _
							 0, _
							 quixieSCREEN
			
			'check for additional process from user function
			Engine_RunMapScript kseMap(iMID).sName & "_onEnterFrame", 0
			
			Engine_Render()
			
			Engine_CheckForMovement()
			
			Engine_SpriteBillboardCheckup() 'this updates the rendering order of sprites
		Loop While kseMap(iMID).bRendering 'while not being paused
		
		If kseMap(iMID).bRender = 1 Then
			'check for pause function
			Engine_RunMapScript kseMap(iMID).sName & "_onPause", 0
		End If
	Loop While kseMap(iMID).bRender 'while not being ended
	
	Engine_RunMapScript kseMap(iMID).sName & "_onUnload", 0
	
	Return iIndex
End Function

''
' (<b>Dev API</b>) Goal is to give the illusion of Billboard concept (2D-in-3D) in DX8.
' This <i>updates</i> the "rendering <i>order</i>" of sprites.
'
' @remarks  None.
' @date 04.29.07
Private Sub Engine_SpriteBillboardCheckup()
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	For i As Integer = 1 To kseMap(iMID).EntitiesCtr - 2
		For i2 As Integer = 2 To kseMap(iMID).EntitiesCtr - 1
			If kseMap(iMID).Entities[i].iY > kseMap(iMID).Entities[i2].iY Then
				'switch the player ID if index is the assigned PLAYER
				If i = kseMap(iMID).iPlayer Then
					kseMap(iMID).iPlayer = i2
					kseMap(iMID).EntityFocus = i2
				ElseIf i2 = kseMap(iMID).iPlayer Then
					kseMap(iMID).iPlayer = i
					kseMap(iMID).EntityFocus = i
				End If
				'place i to TMP
				With kseMap(iMID).Entities[i]
					kseMap(iMID).Entities[0].bHide = .bHide
					kseMap(iMID).Entities[0].bStatic = .bStatic
					kseMap(iMID).Entities[0].bUsed = .bUsed
					
					kseMap(iMID).Entities[0].iAnmt_DownTileMax = .iAnmt_DownTileMax
						fbUDTtoUDT .iAnmt_DownTiles, kseMap(iMID).Entities[0].iAnmt_DownTiles,.iAnmt_DownTileMax
					kseMap(iMID).Entities[0].iAnmt_LeftTileMax = .iAnmt_LeftTileMax
						fbUDTtoUDT .iAnmt_LeftTiles, kseMap(iMID).Entities[0].iAnmt_LeftTiles,.iAnmt_LeftTileMax
					kseMap(iMID).Entities[0].iAnmt_RightTileMax = .iAnmt_RightTileMax
						fbUDTtoUDT .iAnmt_RightTiles, kseMap(iMID).Entities[0].iAnmt_RightTiles,.iAnmt_RightTileMax
					kseMap(iMID).Entities[0].iAnmt_UpTileMax = .iAnmt_UpTileMax
						fbUDTtoUDT .iAnmt_UpTiles, kseMap(iMID).Entities[0].iAnmt_UpTiles,.iAnmt_UpTileMax
					kseMap(iMID).Entities[0].iDirection = .iDirection
					kseMap(iMID).Entities[0].iHeight = .iHeight
					kseMap(iMID).Entities[0].IsAttachedToMap = .IsAttachedToMap
					kseMap(iMID).Entities[0].iSpriteHandlerMax = .iSpriteHandlerMax
						fbUDTtoUDT .iSpriteHandlers, kseMap(iMID).Entities[0].iSpriteHandlers,.iSpriteHandlerMax
					kseMap(iMID).Entities[0].iSpriteIndex = .iSpriteIndex
					kseMap(iMID).Entities[0].iWidth = .iWidth
					kseMap(iMID).Entities[0].iX = .iX
					kseMap(iMID).Entities[0].iY = .iY
					kseMap(iMID).Entities[0].sName = .sName
				End With
				'place i2 to i
				With kseMap(iMID).Entities[i2]
					kseMap(iMID).Entities[i].bHide = .bHide
					kseMap(iMID).Entities[i].bStatic = .bStatic
					kseMap(iMID).Entities[i].bUsed = .bUsed
					kseMap(iMID).Entities[i].iAnmt_DownTileMax = .iAnmt_DownTileMax
						fbUDTtoUDT .iAnmt_DownTiles, kseMap(iMID).Entities[i].iAnmt_DownTiles,.iAnmt_DownTileMax
					kseMap(iMID).Entities[i].iAnmt_LeftTileMax = .iAnmt_LeftTileMax
						fbUDTtoUDT .iAnmt_LeftTiles, kseMap(iMID).Entities[i].iAnmt_LeftTiles,.iAnmt_LeftTileMax
					kseMap(iMID).Entities[i].iAnmt_RightTileMax = .iAnmt_RightTileMax
						fbUDTtoUDT .iAnmt_RightTiles, kseMap(iMID).Entities[i].iAnmt_RightTiles,.iAnmt_RightTileMax
					kseMap(iMID).Entities[i].iAnmt_UpTileMax = .iAnmt_UpTileMax
						fbUDTtoUDT .iAnmt_UpTiles, kseMap(iMID).Entities[i].iAnmt_UpTiles,.iAnmt_UpTileMax
					kseMap(iMID).Entities[i].iDirection = .iDirection
					kseMap(iMID).Entities[i].iHeight = .iHeight
					kseMap(iMID).Entities[i].IsAttachedToMap = .IsAttachedToMap
					kseMap(iMID).Entities[i].iSpriteHandlerMax = .iSpriteHandlerMax
						fbUDTtoUDT .iSpriteHandlers, kseMap(iMID).Entities[i].iSpriteHandlers,.iSpriteHandlerMax
					kseMap(iMID).Entities[i].iSpriteIndex = .iSpriteIndex
					kseMap(iMID).Entities[i].iWidth = .iWidth
					kseMap(iMID).Entities[i].iX = .iX
					kseMap(iMID).Entities[i].iY = .iY
					kseMap(iMID).Entities[i].sName = .sName
				End With
				'place TMP to i2
				With kseMap(iMID).Entities[0]
					kseMap(iMID).Entities[i2].bHide = .bHide
					kseMap(iMID).Entities[i2].bStatic = .bStatic
					kseMap(iMID).Entities[i2].bUsed = .bUsed
					kseMap(iMID).Entities[i2].iAnmt_DownTileMax = .iAnmt_DownTileMax
						fbUDTtoUDT .iAnmt_DownTiles, kseMap(iMID).Entities[i2].iAnmt_DownTiles,.iAnmt_DownTileMax
					kseMap(iMID).Entities[i2].iAnmt_LeftTileMax = .iAnmt_LeftTileMax
						fbUDTtoUDT .iAnmt_LeftTiles, kseMap(iMID).Entities[i2].iAnmt_LeftTiles,.iAnmt_LeftTileMax
					kseMap(iMID).Entities[i2].iAnmt_RightTileMax = .iAnmt_RightTileMax
						fbUDTtoUDT .iAnmt_RightTiles, kseMap(iMID).Entities[i2].iAnmt_RightTiles,.iAnmt_RightTileMax
					kseMap(iMID).Entities[i2].iAnmt_UpTileMax = .iAnmt_UpTileMax
						fbUDTtoUDT .iAnmt_UpTiles, kseMap(iMID).Entities[i2].iAnmt_UpTiles,.iAnmt_UpTileMax
					kseMap(iMID).Entities[i2].iDirection = .iDirection
					kseMap(iMID).Entities[i2].iHeight = .iHeight
					kseMap(iMID).Entities[i2].IsAttachedToMap = .IsAttachedToMap
					kseMap(iMID).Entities[i2].iSpriteHandlerMax = .iSpriteHandlerMax
						fbUDTtoUDT .iSpriteHandlers, kseMap(iMID).Entities[i2].iSpriteHandlers,.iSpriteHandlerMax
					kseMap(iMID).Entities[i2].iSpriteIndex = .iSpriteIndex
					kseMap(iMID).Entities[i2].iWidth = .iWidth
					kseMap(iMID).Entities[i2].iX = .iX
					kseMap(iMID).Entities[i2].iY = .iY
					kseMap(iMID).Entities[i2].sName = .sName
				End With
			End If
		Next
	Next
End Sub

''
' (<b>Dev API</b>) Checks if player pressed a key.
'
' @remarks This is the function that hooks the user input to the game.
Private Sub Engine_CheckForMovement()
	'sentinel for checking if player was obstructed; look at the bottom part of this routine
	Dim oldX As Integer
	Dim oldY As Integer
	
	Dim newDir As kseDIRECTION
	Dim bMoveMap As Byte
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	With kseMap(iMID)
		.bMoveLeft = bMapMoveLeft
		.bMoveRight = bMapMoveRight
		.bMoveUp = bMapMoveUp
		.bMoveDown = bMapMoveDown
		.bRender = bMapRender
		.bRendering = bMapRendering
	End With
	
	'is it valid to check for new movement
	If kseMap(iMID).iMoveMode = kseMAP_MOVE_TILEBASED_MODE _
			And iMapMoveCtr >= iMapMoveCtrMax Then
		'check for new movement
		If kseMap(iMID).bMoveUp Then
			newDir = dirUP
		ElseIf kseMap(iMID).bMoveDown Then
			newDir = dDOWN
		ElseIf kseMap(iMID).bMoveLeft Then
			newDir = dLEFT
		ElseIf kseMap(iMID).bMoveRight Then
			newDir = dRIGHT
		End If
	Else
		bMoveMap = 1
	End If
	
	If newDir <> dNOT_SET Then
		'set new direction of 'player/character'
		Engine_EntityChangeDirection kseMap(iMID).iPlayer, newDir
		'there should be no obstruction
		If Engine_IsTileObstruct(kseMap(iMID).iPlayer, CInt(iMID)) = 0 Then
			'get array count of a direction-animation
			'Print "got new direction:" & newDir
			iMapMoveCtr = 0
			'Select Case newDir
			'	Case dDOWN
			'		iMapMoveCtrMax = UBound(kseEntity(kseEntity(iIdx).iSpriteIndex).iAnmt_DownTiles)
			'	Case dirUP
			'		iMapMoveCtrMax = UBound(kseEntity(kseEntity(iIdx).iSpriteIndex).iAnmt_UpTiles)
			'	Case dLEFT
			'		iMapMoveCtrMax = UBound(kseEntity(kseEntity(iIdx).iSpriteIndex).iAnmt_LeftTiles)
			'	Case dRIGHT
			'		iMapMoveCtrMax = UBound(kseEntity(kseEntity(iIdx).iSpriteIndex).iAnmt_RightTiles)
			'End Select
			iMapMoveCtrMax = kseMap(iMID).MapDimension
		End If
	End If
	
	'set default value
	If bMapFocusingOnEntity = 1 Then
		oldX = kseMap(iMID).iX
		oldY = kseMap(iMID).iY
	Else
		oldX = kseMap(iMID).Entities[kseMap(iMID).iPlayer].iX
		oldY = kseMap(iMID).Entities[kseMap(iMID).iPlayer].iY
	End If
	
	If kseMap(iMID).iMoveMode = kseMAP_MOVE_TILEBASED_MODE _
			And iMapMoveCtr < iMapMoveCtrMax Then 'bMoveMap
		'trace "iMapMoveCtr-" & iMapMoveCtr
		'TILE-BASED-MOVEMENT MODE
		'default and the only working one (for now)
		
		iMapMove = 1 'Math.Round(kseMap(imid).MapDimension / iMapMoveCtrMax)
		iMapMoveCtr = iMapMoveCtr + 1 'iMapMove
		Dim iMapDim As Integer
		iMapDim = kseMap(iMID).MapDimension


		If bMapFocusingOnEntity = 1 Then
			With kseMap(iMID).Entities[kseMap(iMID).iPlayer]
				oldX = kseMap(iMID).iX
				oldY = kseMap(iMID).iY
				
				Select Case kseMap(iMID).Entities[kseMap(iMID).iPlayer].iDirection
					Case dDOWN
						'limit moving of map upwards; entity is trying to move down
						If (iMapDim - .iHeight) < ((iMapDim * kseMap(iMID).iHeight) - (.iY + .iHeight)) Then
							kseMap(iMID).iY = kseMap(iMID).iY - iMapMove
							.iY = .iY + 1
						End If
					Case dirUP
						'limit moving of map downwards; entity is trying to move up
						If kseMap(iMID).iY < ((kseImg(quixieSCREEN).imgHeight - .iHeight) / 2) Then
							kseMap(iMID).iY = kseMap(iMID).iY + iMapMove
							.iY = .iY - 1
						End If
					Case dLEFT
						'limit moving of map rightwards; entity is trying to move left
						If kseMap(iMID).iX < Int((kseImg(quixieSCREEN).imgWidth - .iWidth) / 2) Then
							kseMap(iMID).iX = kseMap(iMID).iX + iMapMove
							.iX = .iX - 1
						End If
					Case dRIGHT
						'limit moving of map leftwards; entity is trying to move right
						If (iMapDim - .iWidth) < ((iMapDim * kseMap(iMID).iWidth) - (.iX + .iWidth)) Then
							kseMap(iMID).iX = kseMap(iMID).iX - iMapMove
							.iX = .iX + 1
						End If
				End Select
			End With

		Else
			With kseMap(iMID).Entities[kseMap(iMID).iPlayer]
'				oldX = .iX
'				oldY = .iY
			
				Select Case kseMap(iMID).Entities[kseMap(iMID).iPlayer].iDirection
					Case dDOWN
						'limit moving of map upwards; entity is trying to move down
						If .iY < (iMapDim * (kseMap(iMID).iHeight - 1)) Then
						'If kseMap(iMID).iY > XXX Then
							.iY = .iY + 1
						End If
					Case dirUP
						'limit moving of map downwards; entity is trying to move up
						If .iY > 0 Then
							.iY = .iY - 1
						End If
					Case dLEFT
						'limit moving of map rightwards; entity is trying to move left
						If .iX > 0 Then
							.iX = .iX - 1
						End If
					Case dRIGHT
						'limit moving of map leftwards; entity is trying to move right
						If (iMapDim - .iWidth) < ((iMapDim * kseMap(iMID).iWidth) - (.iX + .iWidth)) Then
'						If .iX < (iMapDim * (kseMap(iMID).iWidth - 1)) Then
							kseMap(iMID).iX = kseMap(iMID).iX - iMapMove
							.iX = .iX + 1
						End If
				End Select
			End With
		End If
	Else
		'FREE MODE
		'TODO:study how RELSoft's scroll engine works
		'If kseMap(iMID).bMoveUp Then
		'	kseMap(iMID).iY = kseMap(iMID).iY + iMapMove 'kseMAP_MOVE
		'ElseIf kseMap(iMID).bMoveDown Then
		'	kseMap(iMID).iY = kseMap(iMID).iY - iMapMove 'kseMAP_MOVE
		'End If
		'If kseMap(iMID).bMoveLeft Then
		'	kseMap(iMID).iX = kseMap(iMID).iX + iMapMove 'kseMAP_MOVE
		'ElseIf kseMap(iMID).bMoveRight Then
		'	kseMap(iMID).iX = kseMap(iMID).iX - iMapMove 'kseMAP_MOVE
		'End If
	End If

	'animate 'player/character' only when moving
	If bMapFocusingOnEntity = 1 Then
		If oldX = kseMap(iMID).iX And oldY = kseMap(iMID).iY Then
			kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 1
		Else
			kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 0
		End If
	Else
		If oldX = kseMap(iMID).Entities[kseMap(iMID).iPlayer].iX _
				And oldY = kseMap(iMID).Entities[kseMap(iMID).iPlayer].iY Then
			kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 1
		Else
			kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 0
		End If
	End If
End Sub

''
' (<b>Dev API</b>) Executes a function related to map.
'
' @param	fxName - name of function to execute.
' @param	bWarn - ID of map to be checked.
' @remarks Function wrapper for map files. -- it uses <b>Engine_RunScript</b>.
Public Sub Engine_RunMapScript(fxName As String, bWarn As Byte = 1)
	Engine_RunScript sMapName, fxName, , bWarn
End Sub


''
' (<b>Dev API</b>) Returns a sprite-image-buffer of an entity to be displayed in <b>Game:Render</b>.
'
' @param	iEntity - Which entity among the attached entity is to be checked?
' @return   Frame index of the sprite-animation to be rendered.
' @remarks None.
Private Function KSE_GetSpriteAnimation(iEntity As Integer) As Integer
	Dim iEntCtr As Integer
	Dim iSprIdx As Integer
	Dim iFmeCtr As Integer  'animate or not
	Dim iDir As kseDIRECTION
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	
	With kseMap(iMID).Entities[iEntity]
		iDir = .iDirection
		If .bStatic = 0 Then iFmeCtr = 1
		
		If iDir = dDOWN Then
			'get current counter then add one
			iEntCtr = .iAnmt_DownCtr + iFmeCtr
			
			'check for out-of-bounds
			If iEntCtr >= .iAnmt_DownTileMax Then iEntCtr = 0
			
			'set new counter value
			.iAnmt_DownCtr = iEntCtr
			
			'get tile based from counter value
			iSprIdx = .iAnmt_DownTiles[iEntCtr]
		ElseIf iDir = dLEFT Then
			'get current counter then add one
			iEntCtr = .iAnmt_LeftCtr + iFmeCtr
			
			'check for out-of-bounds
			If iEntCtr >= .iAnmt_LeftTileMax Then iEntCtr = 0
			
			'set new counter value
			.iAnmt_LeftCtr = iEntCtr
			
			'get tile based from counter value
			iSprIdx = .iAnmt_LeftTiles[iEntCtr]
		ElseIf iDir = dRIGHT Then
			'get current counter then add one
			iEntCtr = .iAnmt_RightCtr + iFmeCtr
			
			'check for out-of-bounds
			If iEntCtr >= .iAnmt_RightTileMax Then iEntCtr = 0
			
			'set new counter value
			.iAnmt_RightCtr = iEntCtr
			
			'get tile based from counter value
			iSprIdx = .iAnmt_RightTiles[iEntCtr]
		ElseIf iDir = dirUP Then
			'get current counter then add one
			iEntCtr = .iAnmt_UpCtr + iFmeCtr
			
			'check for out-of-bounds
			If iEntCtr >= .iAnmt_UpTileMax Then iEntCtr = 0
			
			'set new counter value
			.iAnmt_UpCtr = iEntCtr
			
			'get tile based from counter value
			iSprIdx = .iAnmt_UpTiles[iEntCtr]
		End If
		
		Return .iSpriteHandlers[iSprIdx]
	End With
End Function

''
' <b>Game:FocusEntity</b> - Gives focus on an entity.
'
' @param	iIndex - Index of the parsed script.
' @return   New index value of the parsed script.
' @remarks  None.
'<div class="KS">
' <b>KonsolScript Users:</b><br>
' <br><b>Description:</b><br>
' Gives game camera's focus on a specific entity.
' <br><br><b>Parameters:</b><br>
' <ul>
'  <li>String <b>entity</b> - instance name of the entity to be focused.
' </ul>
' <b>Usage:</b><br>
' <pre>
'  //provided testMap is a defined map (see Map:New)
'  Game:Render(testMap)
'  //provided that myEntity is an instance of a defined entity (See Game:AttachEntity)
'  Game:FocusEntity("myEntity")
' </pre>
'</div>
Private Function GameFocusEntity(ByRef iIndex As Long) As Integer
	Dim sEntity As String
	
	iIndex = Game_SkipOpenParenthesisAfter("FocusEntity", iIndex)
		
		sEntity = GetStrVal(iIndex, 1, 0)
		
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex)
	'EXECUTE SCRIPT =====================================================================
	Dim bGoCheck As Byte
	Dim iMID As Integer
	iMID = kseMAP_CURRENT_RENDER
	If kseMap(iMID).iMoveMode = kseMAP_MOVE_TILEBASED_MODE Then
		'make sure to finish player's tile-based movement
		If iMapMoveCtr >= iMapMoveCtrMax Then
			bGoCheck = 1
		End If
	Else
		bGoCheck = 1
	End If
	Dim i As Integer
	If bGoCheck = 1 Then
		If sEntity <> "" Then
			bMapFocusingOnEntity = 1
			
			For i = 1 To kseMap(iMID).EntitiesCtr - 1
				If LCase(kseMap(iMID).Entities[i].sName) = LCase(sEntity) Then
					If kseMap(iMID).EntityFocus <> i Then
						'NOTE: this condition should be kept separated with the first 2 condition
						If (kseMap(iMID).iPlayer <> 0 And kseMap(iMID).Entities[kseMap(iMID).iPlayer].bStatic = 1) _
								Or kseMap(iMID).iPlayer = 0 Then
							'change current player only when current player-entity is not moving
							kseMap(iMID).EntityFocus = i
							kseMap(iMID).iX = (-kseMap(iMID).Entities[i].iX) + (kseImg(quixieSCREEN).imgWidth / 2) - (kseMap(iMID).Entities[i].iWidth / 2)
							kseMap(iMID).iY = (-kseMap(iMID).Entities[i].iY) + (kseImg(quixieSCREEN).imgHeight / 2) - (kseMap(iMID).Entities[i].iHeight / 2)
						End If
					End If
					Exit For
				End If
			Next
		Else
			'remove focus to entity
			If kseMap(iMID).EntityFocus > 0 Then
				i = kseMap(iMID).EntityFocus
				kseMap(iMID).iX = (-kseMap(iMID).Entities[i].iX) + (kseImg(quixieSCREEN).imgWidth / 2) - (kseMap(iMID).Entities[i].iWidth / 2)
				kseMap(iMID).iY = (-kseMap(iMID).Entities[i].iY) + (kseImg(quixieSCREEN).imgHeight / 2) - (kseMap(iMID).Entities[i].iHeight / 2)
			End If
			kseMap(iMID).EntityFocus = 0
			
			kseMap(iMID).iCamX = -kseMap(iMID).iX
			kseMap(iMID).iCamY = -kseMap(iMID).iY
			
			bMapFocusingOnEntity = 0
		End If
	End If
	
	Return iIndex
End Function

''
' (<b>Dev API</b>) Returns a tile-image-buffer to be displayed in <b>Game:Render</b>.
'
' @param	iTileIdx - Index of the tile to be checked
' @param	iMID - ID of map to be checked.
' @return   See remarks for better understanding; this will return the <b>z</b>.
' @remarks Guide to this algo <i>(it's driving me crazy!)</i><br>
'DELAY is PROGRAMMED as <b>x / y = z</b> <br>
'<b>x</b> - blitting counter; if it reaches the value of y, add 1 to z <br>
'<b>y</b> - number of times for a single frame to be blitted <br>
'<b>z</b> - current frame being shown <br>
Private Function KSE_GetTileAnimation(iTileIdx As Integer, iMID As Integer) As Integer
	Dim iTileCtr As Integer	  'our z
	Dim iTileCtr2 As Integer  'our x
	Dim iTileIdx2 As Integer
	'get index of "tile-animation array"
	iTileIdx2 = kseMap(iMID).iTileData[iTileIdx].iIdx
	'get index of the current frame from tile-animation to be shown
	iTileCtr2 = kseMap(iMID).iTileData[iTileIdx].iCtr + 1
	
	'get old counter then add one
	iTileCtr = kseTileAnimation(iTileIdx2).iCtr
	
	If iTileCtr2 > kseMap(iMID).iTileData[iTileIdx].iDelay Then
		iTileCtr2 = 0
		'add one to current counter
		iTileCtr = iTileCtr + 1
	End If
	
	'check for out-of-bounds
	If iTileCtr > kseTileAnimation(iTileIdx2).iTiles.Count() Then iTileCtr = 0
	'set new counter value
	kseTileAnimation(iTileIdx2).iCtr = iTileCtr
	
	kseMap(iMID).iTileData[iTileIdx].iCtr = iTileCtr2
	
	'get tile based from counter value
	Dim As Integer iTileIndex = kseTileAnimation(iTileIdx2).iTiles.ItemAt(iTileCtr)
	If iTileIndex < 0  Then
		iTileIndex = kseTileAnimation(iTileIdx2).iTiles.ItemAt(1)
	End If
	
	iTileIndex = iTileIndex - 1
		If iTileIndex < 0  Then iTileIndex = 0
	
	Return iTileIndex
End Function

''
' (<b>Dev API</b>) Game-class' wrapper for skipping open parenthesis.
'
' @param	sWhat - an open parenthesis should be written next to what text?
' @param	iIndex - Index of the parsed script
' @return   New index value of the parsed script.
' @remarks  Benchmark if organizing is worth the speed.
Private Function Game_SkipOpenParenthesisAfter(sWhat As String, ByRef iIndex As Long) As Integer
	cGameTxt = "Game:" & sWhat
	Return KSE_SkipOpenParenthesisAfter(cGameTxt, iIndex)
End Function

''
' (<b>Dev API</b>) Game-class' wrapper for skipping a comma.
'
' @param	sWhat - a comma should be written next to what text?
' @param	iIndex - Index of the parsed script
' @return   New index value of the parsed script.
' @remarks  Benchmark if organizing is worth the speed.
Private Function Game_SkipCommaAfter(sWhat As String, ByRef iIndex As Long) As Integer
	cGameTxt = cGameTxt & sWhat
		iIndex = KSE_SkipCommaAfter(cGameTxt, iIndex)
	cGameTxt = cGameTxt & ", "
	
	Return iIndex
End Function

''
' (<b>Dev API</b>) Game-class' wrapper for skipping close parenthesis.
'
' @param	sWhat - a close parenthesis should be written next to what text?
' @param	iIndex - Index of the parsed script
' @return   New index value of the parsed script.
' @remarks  Benchmark if organizing is worth the speed.
Private Function Game_SkipCloseParenthesisAfter(sWhat As String, ByRef iIndex As Long) As Integer
	cGameTxt = cGameTxt & sWhat
		iIndex = KSE_SkipCloseParenthesisAfter(cGameTxt, iIndex)
	cGameTxt = ""
	Return iIndex
End Function

