//translated this 18.06.10 by mjmendozaIV
#include "../../kage.hpp"
#include "../../kageExterns.hpp"

////
// Parser for KonsolScript's <b>Game</b> class.
//
// This enables KonsolScript to do tile-based game rendering.<br>
// TODO:
// <ul>
//  <li>AttachMap - Adds a map to be used for the game. (is this not the same with Game:Render)
//  <li>Load - loads a game state.
//  <li>Save - saves a game state.
//  <li>Start - runs the game. (Game:Render does the same)
// </ul>

////
// Used by friendly debugging
//
// @return String
// @remarks None.
string /*shared*/ cGameTxt;

////
// Parses and interprets KonsolScript's <b>Game</b> functions.
//
// @param	iIndex - Index of the parsed script
// @return   New index value of the parsed script.
// @remarks Needs more research and refining.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// No need to make game buffers <i>(There should only be one instance of a game per session].</i><br>
// Game class currently contains the functions:
// <ul>
// <li><b><a href="kse_Game.GameAttachEntity.html">AttachEntity</a></b> - Adds an entity to the game.
// <li><b><a href="kse_Game.GameEntitySetX.html">EntitySetX</a></b> - Sets an entity's X location.
// <li><b><a href="kse_Game.GameEntitySetY.html">EntitySetY</a></b> - Sets an entity's Y location.
// <li><b><a href="kse_Game.GameEntityGetX.html">EntityGetX</a></b> - Returns an entity's X location.
// <li><b><a href="kse_Game.GameEntityGetY.html">EntityGetY</a></b> - Returns an entity's Y location.
// <li><b><a href="kse_Game.GameFocusEntity.html">FocusEntity</a></b> - Gives focus on an entity.
// <li><b><a href="kse_Game.GameGetCameraX.html">GetCameraX</a></b> - Returns the game camera's X location.
// <li><b><a href="kse_Game.GameGetCameraY.html">GetCameraY</a></b> - Returns the game camera's Y location.
// <li><b><a href="kse_Game.GameSetCameraX.html">SetCameraX</a></b> - Sets the game camera's X location.
// <li><b><a href="kse_Game.GameSetCameraY.html">SetCameraY</a></b> - Sets the game camera's Y location.
// <li><b><a href="kse_Game.GameSetPlayer.html">SetPlayer</a></b> - Set which entity is to be user-controlled.
// <li><b><a href="kse_Game.GamePause.html">Pause</a></b> - Pauses a game from rendering.
// <li><b><a href="kse_Game.GameResume.html">Resume</a></b> - Resumes a game from pausing.
// <li><b><a href="kse_Game.GameStop.html">Stop</a></b> - Stops the game.
// <!--  TO DO -->
// <!-- li><b><a href="kse_Game.GameAttachMap.html">AttachMap</a></b> - Adds a map to be used for the game. -->
// <!-- li><b><a href="kse_Game.GameLoad.html">Load</a></b> - Loads a game state. -->
// <!-- li><b><a href="kse_Game.GameSave.html">Save</a></b> - Saves a game state. -->
// <!-- li><b><a href="kse_Game.GameStart.html">Start</a></b> - Runs the game. -->
// </ul>
//</div>
unsigned long SynGame(unsigned long &iIndex) {
	iIndex = GetNearestTokenIndex(kseCOLON, "Game", iIndex);   //look for		:
	iIndex = GetNextIndex(iIndex);
	if (kseToken[iIndex].sToken == "Render") {
		return GameRender(iIndex);
	//} else if (kseToken[iIndex].sToken == "Play") {
		//	return GameLoad(iIndex);
	} else if (kseToken[iIndex].sToken == "SetCameraX") {
			return GameSetCameraX(iIndex);
	} else if (kseToken[iIndex].sToken == "SetCameraY") {
			return GameSetCameraY(iIndex);
	} else if (kseToken[iIndex].sToken == "GetCameraX") {
			return GameGetCameraX(iIndex);
	} else if (kseToken[iIndex].sToken == "GetCameraY") {
			return GameGetCameraY(iIndex);
	//} else if (kseToken[iIndex].sToken == "Degenerate") {
		//	return GameDegenerate(iIndex);
	} else if (kseToken[iIndex].sToken == "Resume") {
		return GameResume(iIndex);
	} else if (kseToken[iIndex].sToken == "Pause") {
		return GamePause(iIndex);
	} else if (kseToken[iIndex].sToken == "Stop") {
		return GameStop(iIndex);
	} else if (kseToken[iIndex].sToken == "EntitySetX") {
		return GameEntitySetX(iIndex);
	} else if (kseToken[iIndex].sToken == "EntitySetY") {
		return GameEntitySetY(iIndex);
	} else if (kseToken[iIndex].sToken == "EntityGetX") {
		return GameEntityGetX(iIndex);
	} else if (kseToken[iIndex].sToken == "EntityGetY") {
		return GameEntityGetY(iIndex);
	} else if (kseToken[iIndex].sToken == "AttachEntity") {
		return GameAttachEntity(iIndex);
	//} else if (kseToken[iIndex].sToken == Case "DetachEntity") {
		//	return GameDetachEntity(iIndex);
	} else if (kseToken[iIndex].sToken == "SetPlayer") {
		return GameSetPlayer(iIndex);
	} else if (kseToken[iIndex].sToken == "FocusEntity") {
		return GameFocusEntity(iIndex);
	} else {
		Engine_Message(KSE_GetErrFileSrc(iIndex) + "(  " + KSE_GetToken(iIndex) + "  ) is not in Game Class.", "ERROR AT LINE " + Str(kseToken[iIndex].iLine), 1);
	}
	//to be implemented once KonsolScript is ready for production release
	//Return GetNearestTokenIndex(kseSEMICOLON, kseCLOSE_PARENTHESIS, SynMap)   //look for		;
	return ULONG_MAX;
}

////
// <b>Game:SetCameraX</b> - Sets the game camera's X location.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Sets the X location of the camera of the current map being rendered. Can be used for customized cut-scene.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>Number <b>value</b> - desired new X location.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  Game:SetCameraX(100)
// </pre>
//</div>
unsigned long GameSetCameraX(unsigned long &iIndex) {
	int iVal;
	
	iIndex = Game_SkipOpenParenthesisAfter("SetCameraX", iIndex);
		
		iVal = KSE_GetTokenNumValueOf(iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("value", iIndex);
	//EXECUTE SCRIPT =====================================================================
	kseMap[kseMAP_CURRENT_RENDER].iCamX = iVal;
	kseMap[kseMAP_CURRENT_RENDER].iX = -kseMap[kseMAP_CURRENT_RENDER].iCamX;
	
	return iIndex;
}

////
// <b>Game:SetCameraY</b> - Sets the game camera's Y location.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Sets the Y location of the camera of the current map being rendered. Can be used for customized cut-scene.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>Number <b>value</b> - desired new Y location.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  Game:SetCameraY(100)
// </pre>
//</div>
unsigned long GameSetCameraY(unsigned long &iIndex) {
	int iVal;
	
	iIndex = Game_SkipOpenParenthesisAfter("SetCameraY", iIndex);
		
		iVal = KSE_GetTokenNumValueOf(iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("value", iIndex);
	//EXECUTE SCRIPT =====================================================================
	kseMap[kseMAP_CURRENT_RENDER].iCamY = iVal;
	kseMap[kseMAP_CURRENT_RENDER].iY = -kseMap[kseMAP_CURRENT_RENDER].iCamY;
	
	return iIndex;
}

////
// <b>Game:GetCameraX</b> - Returns the game camera's X location.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Returns the X location of the camera. Can be used for customized cut-scene animation.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>Number <b>return</b> - a <b>Number</b> variable for return value.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  Var:Number xLoc;
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  Game:GetCameraX(xLoc)
// </pre>
//</div>
unsigned long GameGetCameraX(unsigned long &iIndex) {
	int iIdx;
	
	iIndex = Game_SkipOpenParenthesisAfter("GetCameraX", iIndex);
		
		iIndex = GetNextIndex(iIndex);
		iIdx = iIndex;
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex);
	//EXECUTE SCRIPT =====================================================================
	SetNumValue(iIdx, CDbl(kseMap[kseMAP_CURRENT_RENDER].iCamX));
	
	return iIndex;
}

////
// <b>Game:GetCameraY</b> - Returns the game camera's Y location.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Returns the Y location of the camera. Can be used for customized cut-scene animation.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>Number <b>return</b> - a <b>Number</b> variable for return value.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  Var:Number yLoc;
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  Game:GetCameraY(yLoc)
// </pre>
//</div>
unsigned long GameGetCameraY(unsigned long &iIndex) {
	int iIdx;
	
	iIndex = Game_SkipOpenParenthesisAfter("GetCameraY", iIndex);
		
		iIndex = GetNextIndex(iIndex);
		iIdx = iIndex;
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex);
	//EXECUTE SCRIPT =====================================================================
	SetNumValue(iIdx, CDbl(kseMap[kseMAP_CURRENT_RENDER].iCamY));
	
	return iIndex;
}

unsigned long GameSetPlayer(unsigned long &iIndex) {
	string sEntity;
	int iMID;
	iMID = kseMAP_CURRENT_RENDER;
	
	iIndex = Game_SkipOpenParenthesisAfter("SetPlayer", iIndex);
		
		sEntity = KSE_GetTokenStrValueOf(iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex);
	//EXECUTE SCRIPT =====================================================================
	bool bGoCheck;
	if (kseMap[iMID].iMoveMode == kseMAP_MOVE_TILEBASED_MODE) {
		if (iMapMoveCtr >= iMapMoveCtrMax) {
			//make sure to finish player's tile-based movement
			bGoCheck = true;
		}
	} else {
		bGoCheck = true;
	}
	
	if (bGoCheck == true) {
		//change current player when player-entity is not moving
		for (int i = 1; i < kseMap[iMID].EntitiesCtr - 1; ++i) {
			if (LCase(kseMap[iMID].Entities[i].sName) == LCase(sEntity)) {
				if (kseMap[iMID].iPlayer != i) {
					//NOTE: this condition should be kept separated with the first 2 condition
					if ((kseMap[iMID].iPlayer != 0 && kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic == true)
							|| kseMap[iMID].iPlayer == 0) {
						//change current player when player-entity is not moving
						kseMap[iMID].iPlayer = i;
					}
				}
				break;
			}
		}
	}
	
	
	return iIndex;

}

////
// <b>Game:EntitySetX</b> - Relocates entity's <b>x</b> location.
//
// @param	iIndex - Index of the parsed script
// @return   New index value of the parsed script.
// @remarks None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Relocates entity's <b>x</b> location.<br>
// Please note, that the value should have no decimal value -- assigning decimals will be rounded.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>Number <b>value</b> - desired new <b>x</b> value.
//  <li>string <b>entity_name</b> - name of the entity you want to move.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  //provided myEntity is a defined entity
//  Game:AttachEntity(myEntity, "creek") //attach the entity
//  Game:EntitySetX(100, "creek")		//set new x location
// </pre>
//</div>
unsigned long GameEntitySetX(unsigned long &iIndex) {
	string sEntity;
	int iMID;
	int value;
	iMID = kseMAP_CURRENT_RENDER;
	
	iIndex = Game_SkipOpenParenthesisAfter("EntitySetX", iIndex);
	
		value = KSE_GetTokenNumValueOf(iIndex);
	
	iIndex = Game_SkipCommaAfter("value", iIndex);
	
		sEntity = KSE_GetTokenStrValueOf(iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex);
	//EXECUTE SCRIPT ====================================================
	bool bGoCheck;
	if (kseMap[iMID].iMoveMode == kseMAP_MOVE_TILEBASED_MODE) {
		//make sure to finish player's tile-based movement
		if (iMapMoveCtr >= iMapMoveCtrMax) {
			bGoCheck = true;
		}
	} else {
		bGoCheck = false;
	}
	
	if (bGoCheck == true) {
		//change current player when player-entity is not moving
		for (int i = 1; i < kseMap[iMID].EntitiesCtr - 1; ++i) {
			if (LCase(kseMap[iMID].Entities[i].sName) == LCase(sEntity)) {
				//NOTE: this condition should be kept separated with the first 2 condition
				if ((kseMap[iMID].iPlayer != 0 && kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic == true)
						|| kseMap[iMID].iPlayer == 0) {
					int tile_center_x = (kseMap[iMID].MapDimension / 2) - (kseMap[iMID].Entities[i].iWidth / 2);
					int value_excess = value % kseMap[iMID].MapDimension;
					
					value = value - value_excess;
					
					if (value_excess > tile_center_x) {
						value = value + tile_center_x;
					}
					
					kseMap[iMID].Entities[i].iX = value;
					
					break;
				}
			}
		}
	}
	
	return iIndex;
}

////
// <b>Game:EntitySetY</b> - Relocates entity's <b>y</b> location.
//
// @param	iIndex - Index of the parsed script
// @return   New index value of the parsed script.
// @remarks None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Relocates entity's <b>y</b> location.<br>
// Please note, that the value should have no decimal value -- assigning decimals will be rounded.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>Number <b>value</b> - desired new <b>y</b> value.
//  <li>string <b>entity_name</b> - name of the entity you want to move.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  //provided myEntity is a defined entity
//  Game:AttachEntity(myEntity, "creek") //attach the entity
//  Game:EntitySetY(100, "creek")		//set new y location
// </pre>
//</div>
unsigned long GameEntitySetY(unsigned long &iIndex) {
	string sEntity;
	int iMID;
	int value;
	iMID = kseMAP_CURRENT_RENDER;
	
	iIndex = Game_SkipOpenParenthesisAfter("EntitySetY", iIndex);
		
		value = KSE_GetTokenNumValueOf(iIndex);
	
	iIndex = Game_SkipCommaAfter("value", iIndex);
	
		sEntity = KSE_GetTokenStrValueOf(iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex);
	//EXECUTE SCRIPT =====================================================================
	bool bGoCheck;
	if (kseMap[iMID].iMoveMode == kseMAP_MOVE_TILEBASED_MODE) {
		//make sure to finish player's tile-based movement
		if (iMapMoveCtr >= iMapMoveCtrMax) {
			bGoCheck = true;
		}
	} else {
		bGoCheck = false;
	}
	
	if (bGoCheck == true) {
		//change current player when player-entity is not moving
		for (int i = 1; i < kseMap[iMID].EntitiesCtr - 1; ++i) {
			if (LCase(kseMap[iMID].Entities[i].sName) == LCase(sEntity)) {
				//NOTE: this condition should be kept separated with the first 2 condition
				if ((kseMap[iMID].iPlayer != 0 && kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic == true)
						|| kseMap[iMID].iPlayer == 0) {
					int tile_center_y = (kseMap[iMID].MapDimension / 2) - (kseMap[iMID].Entities[i].iHeight / 2);
					int value_excess = value % kseMap[iMID].MapDimension;
					
					value = value - value_excess;
					
					if (value_excess > tile_center_y) {
						value = value + tile_center_y;
					}
					
					kseMap[iMID].Entities[i].iY = value;
					//kseMap[iMID].Entities[i].iY = (Int(value / kseMap[iMID].MapDimension) * kseMap[iMID].MapDimension) - kseMap[iMID].Entities[i].iHeight;
					break;
				}
			}
		}
	}
	
	return iIndex;
}

////
// <b>Game:EntityGetX</b> - Returns the X location of an entity.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Returns the Y location of an entity.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>string <b>entity</b> - instance name of the entity to be attached.
//  <li>Number <b>return</b> - Number variable to place the value.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  Var:Number myYloc;
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  //provided that myEntity is an instance of a defined entity (See Game:AttachEntity)
//  Game:EntityGetY("myEntity", myYloc)
// </pre>
//</div>
unsigned long GameEntityGetX(unsigned long &iIndex) {
	string sEntity;
	int iRet;
	int iMID = kseMAP_CURRENT_RENDER;
	
	iIndex = Game_SkipOpenParenthesisAfter("EntityGetX", iIndex);
		
		sEntity = KSE_GetTokenStrValueOf(iIndex);
	
	iIndex = Game_SkipCommaAfter("entity", iIndex);
	
		iIndex = GetNextIndex(iIndex);
		iRet = iIndex;
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex);
	//EXECUTE SCRIPT =====================================================================
	for (int i = 1; i < kseMap[iMID].EntitiesCtr - 1; ++i) {
		if (LCase(kseMap[iMID].Entities[i].sName) == LCase(sEntity)) {
			SetNumValue(iRet, CDbl(kseMap[iMID].Entities[i].iX));
			break;
		}
	}
	
	return iIndex;
}

////
// <b>Game:EntityGetY</b> - Returns the Y location of an entity.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Returns the Y location of an entity.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>string <b>entity</b> - instance name of the entity to be attached.
//  <li>Number <b>return</b> - Number variable to place the value.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  Var:Number myYloc;
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  //provided that myEntity is an instance of a defined entity (See Game:AttachEntity)
//  Game:EntityGetY("myEntity", myYloc)
// </pre>
//</div>
unsigned long GameEntityGetY(unsigned long &iIndex) {
	string sEntity;
	int iRet;
	int iMID = kseMAP_CURRENT_RENDER;
	
	iIndex = Game_SkipOpenParenthesisAfter("EntityGetY", iIndex);
		
		sEntity = KSE_GetTokenStrValueOf(iIndex);
	
	iIndex = Game_SkipCommaAfter("entity", iIndex);
	
		iIndex = GetNextIndex(iIndex);
		iRet = iIndex;
	
	iIndex = Game_SkipCloseParenthesisAfter("return", iIndex);
	//EXECUTE SCRIPT =====================================================================
	for (int i = 1; i < kseMap[iMID].EntitiesCtr - 1; ++i) {
		if (LCase(kseMap[iMID].Entities[i].sName) == LCase(sEntity)) {
			SetNumValue(iRet, CDbl(kseMap[iMID].Entities[i].iY));
			break;
		}
	}
	
	return iIndex;
}

void Engine_RunScript(string sScriptFile, string fxName, string sFileExt, bool bWarn) {
	unsigned int i;
	for (i = 0; i < iFunctionCtr; ++i) {
		if (kseFunction[i].fName == fxName
				&& (stringToUpper(kseSourceFiles[kseToken[kseFunction[i].fStartIndex].iSrc]) == stringToUpper(sScriptFile + sFileExt)
				|| InStr(1, stringToUpper(kseSourceFiles[kseToken[kseFunction[i].fStartIndex].iSrc]), stringToUpper(kseSLASH + sScriptFile + sFileExt)) > 0)) {
			kseFunction[i].RunScript();
			break;
		}
	}
	
	if (bWarn) {
		if (i > iFunctionCtr) { Engine_Log("Warning: (  function " + fxName + "  ) of (  " + sScriptFile + "  ) does not exist."); }
	}
}

bool ReDim_MapEntity(int new_list_count, unsigned long map_idx) {
	if (map_idx > iMapCtr) { return false; } //failed
	int max_count;
	int i;
	int old_list_count = kseMap[map_idx].EntitiesCtr;
	
	//transfer the data to temp
	vector<typeEntity> tmpEnt;// = CAllocate(old_list_count * sizeof(typeEntity));
	typeEntity l_foo;
	tmpEnt.push_back(l_foo);
		for (i = 0; i < old_list_count - 1; ++i) {
			if (i > 0) { //0 is for tmp
				tmpEnt.push_back(kseMap[map_idx].Entities[i]);
			}
		}
	kseMap[map_idx].Entities.clear();
	//get max list count
		max_count = IIf(new_list_count > old_list_count, new_list_count, old_list_count);
		//kseMap[map_idx].EntitiesCtr = max_count;
	
	//allocate new size
		kseMap[map_idx].Entities.push_back(l_foo);
		//kseMap[map_idx].Entities = CAllocate((max_count+1)*sizeof(typeEntity)); //always add extra for tmp
		//if (kseMap[map_idx].Entities == 0) { return false; }
	//put back data to source
	for (i = 0; i < max_count - 1; ++i) { // last index has no previous value
		if (i <= old_list_count - 1 && i > 0) {
			//kseMap[map_idx].Entities[i] = tmpEnt[i];
			kseMap[map_idx].Entities.push_back(tmpEnt[i]);
		}
	}
	return true;
}

bool ReDim_MapEntityName(int new_list_count, unsigned long map_idx) {
	if (map_idx > iMapCtr) { return false; } //failed
	int max_count;
	int i;
	int old_list_count = kseMap[map_idx].EntitiesCtr;
	
	//transfer the data to temp
	vector<string> tmpEnt;
	tmpEnt.push_back("");
	//[QuixiePP] check faster way to copy vector content
		for (i = 0; i < old_list_count - 1; ++i) {
			if (i > 0) { //0 is for tmp
				tmpEnt.push_back(kseMap[map_idx].EntitiesName[i]);
			}
		}
	kseMap[map_idx].EntitiesName.clear();
	//get max list count
	max_count = IIf(new_list_count > old_list_count, new_list_count, old_list_count);
	
	//allocate new size
		//kseMap[map_idx].EntitiesName = Callocate((max_count+1)*sizeof(String)) //always add extra for tmp
		
		//if (kseMap[map_idx].EntitiesName.size() == 0) { return false; }
	
	//put back data to source
	for (i = 0; i < max_count - 1; ++i) { // start i to 1 since 0 is for tmp purposes
		if (i <= old_list_count - 1 && i > 0) {
			//kseMap[map_idx].EntitiesName[i] = tmpEnt[i];
			kseMap[map_idx].EntitiesName.push_back(tmpEnt[i]);
		}
	}
	return true;
}

////
// <b>Game:AttachEntity</b> - Adds an entity to the game.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Creates an instance from a defined entity and attaches it to the game.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>string <b>entity</b> - instance name of the entity to be attached.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  //provided that myEntity is an instance of a defined entity (See Entity:New)
//  Game:AttachEntity(myEntity, "myEntityInstance")
//  //from then on, accessing the attached entity will be by its name, myEntityInstance
// </pre>
//</div>
unsigned long GameAttachEntity(unsigned long &iIndex) {
	int iEntity;
	string sName;
	unsigned long iMID = kseMAP_CURRENT_RENDER;
	
	iIndex = Game_SkipOpenParenthesisAfter("AttachEntity", iIndex);
		
		iEntity = KSE_GetTokenNumValueOf(iIndex);
		
	iIndex = Game_SkipCommaAfter("entity", iIndex);
	
		sName = KSE_GetTokenStrValueOf(iIndex);
		
	iIndex = Game_SkipCloseParenthesisAfter("name", iIndex);
	//EXECUTE SCRIPT ====================================================
	//vector<typeEntity> kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr];

	//NOTE: these statements are meant to be separated into 2 ifs
	if (iEntity < 1) {
		Engine_Message(KSE_GetErrFileSrc(iIndex) + "Make sure entity file was loaded.", "LOGIC ERROR AT LINE " + kseToken[iIndex].iLine, 1);
	}
	
	if (iMID < 1 || iMID > iMapCtr) {
	   Engine_ERR_NO_MAP(iIndex);
	} else if (kseMap[iMID].bUsed == false) {
	   Engine_ERR_NO_MAP(iIndex);
	}
	if (ReDim_MapEntity(kseMap[iMID].EntitiesCtr+1, iMID) == 0) { Engine_ERR_NO_MAP(iIndex); }
	
	
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].bHide = kseEntity[iEntity].bHide;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].bStatic = true;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].bUsed = true;

//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_LeftTileMax = kseEntity[iEntity].iAnmt_LeftTileMax;
//			fbUDTtoUDT(kseEntity[iEntity].iAnmt_LeftTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_LeftTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_LeftTileMax);
			std::vector<int>::iterator it;
			it = kseEntity[iEntity].iAnmt_LeftTiles.begin();
				kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_LeftTiles.assign(it, kseEntity[iEntity].iAnmt_LeftTiles.end());
//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_LeftCtr = kseEntity[iEntity].iAnmt_LeftCtr
//			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_LeftTiles() = kseEntity[iEntity].iAnmt_LeftTiles()
		
//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_RightTileMax = kseEntity[iEntity].iAnmt_RightTileMax;
//			fbUDTtoUDT(kseEntity[iEntity].iAnmt_RightTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_RightTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_RightTileMax);
			it = kseEntity[iEntity].iAnmt_RightTiles.begin();
			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_RightTiles.assign(it, kseEntity[iEntity].iAnmt_RightTiles.end());
			//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_RightCtr = kseEntity[iEntity].iAnmt_RightCtr
//			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_RightTiles() = kseEntity[iEntity].iAnmt_RightTiles()
		
//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_DownTileMax = kseEntity[iEntity].iAnmt_DownTileMax;
//		   fbUDTtoUDT(kseEntity[iEntity].iAnmt_DownTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_DownTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_DownTileMax);
			it = kseEntity[iEntity].iAnmt_DownTiles.begin();
			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_DownTiles.assign(it, kseEntity[iEntity].iAnmt_DownTiles.end());
//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_DownCtr = kseEntity[iEntity].iAnmt_DownCtr
//			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_DownTiles() = kseEntity[iEntity].iAnmt_DownTiles()
		
//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_UpTileMax = kseEntity[iEntity].iAnmt_UpTileMax;
//			fbUDTtoUDT(kseEntity[iEntity].iAnmt_UpTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_UpTiles, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_UpTileMax);
			it = kseEntity[iEntity].iAnmt_UpTiles.begin();
			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_UpTiles.assign(it, kseEntity[iEntity].iAnmt_UpTiles.end());
//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_UpCtr = kseEntity[iEntity].iAnmt_UpCtr
//			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iAnmt_UpTiles() = kseEntity[iEntity].iAnmt_UpTiles()

		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iDirection = kseEntity[iEntity].iDirection;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iHeight = kseEntity[iEntity].iHeight;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].IsAttachedToMap = true;
//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iSpriteHandlerMax  = kseEntity[iEntity].iSpriteHandlerMax;
//			fbUDTtoUDT(kseEntity[iEntity].iSpriteHandlers, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iSpriteHandlers, kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iSpriteHandlerMax);
			it = kseEntity[iEntity].iSpriteHandlers.begin();
			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iSpriteHandlers.assign(it, kseEntity[iEntity].iSpriteHandlers.end());
			//		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iSpriteHandlers() = kseEntity[iEntity].iSpriteHandlers();
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iSpriteIndex = kseEntity[iEntity].iSpriteIndex;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iWidth = kseEntity[iEntity].iWidth;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iX = kseEntity[iEntity].iX;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].iY = kseEntity[iEntity].iY;
		kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].sName = sName;
	
	
//	vector<string> kseMap[iMID].EntitiesName[kseMap[iMID].EntitiesCtr];
	if (ReDim_MapEntityName(kseMap[iMID].EntitiesCtr+1, iMID) == 1) {
		kseMap[iMID].EntitiesName[kseMap[iMID].EntitiesCtr] = sName;
	}
	
	++kseMap[iMID].EntitiesCtr;
	
	//if (kseEntity[iEntity].IsAttachedToMap == true) { Engine_RelocateEntityOnMap(iEntity); }
	//kseEntity[iEntity].IsAttachedToMap = true;
	//Engine_LocateEntityOnMap(iEntity, iMid);
	
	return iIndex;
}

//TODO: NOT DONE YET
unsigned long GameDetachEntity(unsigned long &iIndex) {
	string sEntity;
	int iMID;
	iMID = kseMAP_CURRENT_RENDER;
	
	iIndex = Game_SkipOpenParenthesisAfter("DetachEntity", iIndex);
	
		sEntity = KSE_GetTokenStrValueOf(iIndex);

	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex);
	//EXECUTE SCRIPT ====================================================
//	vector<int> iMapEntityHandlers[iMapEntityHandlersCtr];
//	iMapEntityHandlers[iMapEntityHandlersCtr] = sEntity;
//	++iMapEntityHandlersCtr;
	for (int i = 0; i < 10; ++i) { //TODO: Edit me
		//kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].bHide = kseEntity[i].bHide;
		if (kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].sName == sEntity) {
			kseMap[iMID].Entities[kseMap[iMID].EntitiesCtr].bUsed = false;
			//kseEntity[iEntity].IsAttachedToMap = false;
		}
	}
		
	return iIndex;
}

unsigned long GamePause(unsigned long &iIndex) {
	iIndex = Game_SkipOpenParenthesisAfter("Pause", iIndex);
	
	//	iMID = KSE_GetTokenNumValueOf(iIndex);
		
	iIndex = Game_SkipCloseParenthesisAfter("(", iIndex);
	//EXECUTE SCRIPT ====================================================
	//iMID = kseMAP_CURRENT_RENDER
	kseMap[kseMAP_CURRENT_RENDER].bRendering = false;
	bMapRendering = false;
	
	return iIndex;
}

unsigned long GameResume(unsigned long &iIndex) {
	iIndex = Game_SkipOpenParenthesisAfter("Resume", iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("(", iIndex);
	//EXECUTE SCRIPT ====================================================
	kseMap[kseMAP_CURRENT_RENDER].bRendering = true;
	bMapRendering = true;
	
	return iIndex;
}

unsigned long GameStop(unsigned long &iIndex) {
	iIndex = Game_SkipOpenParenthesisAfter("Stop", iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("(", iIndex);
	//EXECUTE SCRIPT ====================================================
	//iMID = kseMAP_CURRENT_RENDER
	kseMap[kseMAP_CURRENT_RENDER].bRendering = false;
	bMapRendering = false;
	
	kseMap[kseMAP_CURRENT_RENDER].bRender = false;
	bMapRender = false;
	
	return iIndex;
}

////
// <b>Game:Render</b> - Runs the game based on the map to be assigned.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Runs the game based on the assigned map.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>Number <b>map</b> - the ID of a generated, defined map.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
// </pre>
//</div>
unsigned long GameRender(unsigned long &iIndex) {
	int iLyr;
	int iWid;
	int iHgt;
	int iX;
	int iY;
	int iMID;
	
	iIndex = Game_SkipOpenParenthesisAfter("Render", iIndex);
	
		iMID = KSE_GetTokenNumValueOf(iIndex);
	
	iIndex = Game_SkipCloseParenthesisAfter("map", iIndex);
	//EXECUTE SCRIPT =====================================================================
	kseMAP_CURRENT_RENDER = iMID; //register current map
	
	Engine_NormalizeEntityLocation();
	//hook Map to engine
		kseMap[iMID].bRender = true;
		kseMap[iMID].bRendering = true;
	//hook control on engine
		bMapRender = true;
		bMapRendering = true;
	
	int iCtr;
	int i;
	int iIdx;
	int iIdx2;
	int iTile;
	int iLyrPicHdlr;
	
	Engine_RunMapScript(sMapName + "_onLoad", 0);
	do {
		do {
			iCtr = 0;
			Engine_CLS();
			for (iLyr = 0; i < kseMap[iMID].iLayers - 1; ++i) {
				//skip tile rendering if entity-layer
				if (iLyr == kseMap[iMID].iLayerObstruct - 1 ) {
					iObsLayer = kseMap[iMID].iLayerObstruct; //for use in Engine_IsTileObstruct
					//processing of obstruction data is when entity moves
					//---------------------------------------------------
					//resume counting
					//just add number of entire tiles in a layer
					iCtr = iCtr + kseMap[iMID].iHeight * kseMap[iMID].iWidth;
				} else if (iLyr == kseMap[iMID].iLayerTrigger - 1) {
					iTrgLayer = kseMap[iMID].iLayerTrigger; //for use in Engine_IsTileObstruct
					//processing of trigger data is when entity moves
					//---------------------------------------------------
					//resume counting
					//just add number of entire tiles in a layer
					iCtr = iCtr + kseMap[iMID].iHeight * kseMap[iMID].iWidth;
				//skip tile rendering if entity-layer
				} else if (iLyr == kseMap[iMID].iLayerEntity - 1) {
					//render entity instead
					//start with 1; 0 is for //tmp// purposes
					for (i = 1; i < kseMap[iMID].EntitiesCtr - 1; ++i) {
						
							if (kseMap[iMID].Entities[i].bUsed == true && kseMap[iMID].Entities[i].bHide == false && kseMap[iMID].Entities[i].IsAttachedToMap == true) {
								//get description
								iIdx2 = KSE_GetSpriteAnimation(i);
								//--> from old GameRender iIdx = KSE_GetSpriteAnimation(CInt(i))
								//blit entity to Game's screen
								Engine_BlitToRegion(iIdx2,
												 CInt(kseMap[iMID].Entities[i].iX + abs(kseMap[iMID].MapDimension / 2) - (kseMap[iMID].Entities[i].iWidth / 2)),
												 CInt(kseMap[iMID].Entities[i].iY + kseMap[iMID].MapDimension - kseMap[iMID].Entities[i].iHeight),
												 CInt(kseImg[iIdx].imgWidth),
												 CInt(kseImg[iIdx].imgHeight),
												 1,
												 kseMap[iMID].Map,
												 CInt(kseImg[kseMap[iMID].Map].imgWidth),
												 CInt(kseImg[kseMap[iMID].Map].imgHeight));
							}
						
					}
					//entity layer does not have anything to do with the tiles counting...
					//just add number of entire tiles in a layer
					iCtr = iCtr + kseMap[iMID].iHeight * kseMap[iMID].iWidth;
					//===============================================
				} else { //this is the ordinary/normal tile layer used for BG/FG
					for (iHgt = 1; iHgt < kseMap[iMID].iHeight; ++iHgt) {
						for (iWid = 1; iWid < kseMap[iMID].iWidth; ++iWid) {
							//get the tile index
							iTile = kseMap[iMID].iTileData[iCtr].iIdx;
							if (iTile > 0) { //check if not null
								//locate x and y
								iX = (iWid - 1) * kseMap[iMID].MapDimension;
								iY = (iHgt - 1) * kseMap[iMID].MapDimension;
								//#doing this first before checking for out-of-bounds will
								//  make sure that animated tile is in sync
								
								//get the picture index of an animated tile only; static tiles where pre-drawn on MapGenerate
								if (kseMap[iMID].iTileData[iCtr].bStatic == false) {
									iIdx = KSE_GetTileAnimation(CInt(iCtr), iMID);
									iIdx = kseMap[iMID].iTileImgHdlr[iIdx];
									//draw picture to layer
									iLyrPicHdlr = kseMap[iMID].iLayer[iLyr];
									Engine_BlitToRegion(iIdx,
													 CInt(iX),
													 CInt(iY),
													 CInt(kseImg[iIdx].imgWidth),
													 CInt(kseImg[iIdx].imgHeight),
													 kseMap[iMID].iTileData[iCtr].bTBlit,
													 iLyrPicHdlr,
													 CInt(kseImg[iLyrPicHdlr].imgWidth),
													 CInt(kseImg[iLyrPicHdlr].imgHeight));
								}
							}

							//do the counting; start counting from 0
							++iCtr;
							if (iCtr > (kseMap[iMID].iLayers * kseMap[iMID].iHeight * kseMap[iMID].iWidth)) { iCtr = 0; }
						}
					}
					//draw the whole layer (without the animating tiles)
					iLyrPicHdlr = kseMap[iMID].iLayer[iLyr];
					//draw layer to map
					Engine_BlitToRegion(iLyrPicHdlr,
									 0,
									 0,
									 CInt(kseImg[iLyrPicHdlr].imgWidth),
									 CInt(kseImg[iLyrPicHdlr].imgHeight),
									 1,
									 kseMap[iMID].Map,
									 CInt(kseImg[kseMap[iMID].Map].imgWidth),
									 CInt(kseImg[kseMap[iMID].Map].imgHeight));
				}
			}
			
			if (kseMap[iMID].EntityFocus != 0) {
				//remember to follow the entity being focused
				int iEntFocus = kseMap[iMID].EntityFocus;
				kseMap[iMID].iX = CInt((-kseMap[iMID].Entities[iEntFocus].iX) + (kseImg[quixieSCREEN].imgWidth / 2) - (kseMap[iMID].Entities[iEntFocus].iWidth / 2));
				kseMap[iMID].iY = CInt((-kseMap[iMID].Entities[iEntFocus].iY) + (kseImg[quixieSCREEN].imgHeight / 2) - (kseMap[iMID].Entities[iEntFocus].iHeight / 2));
			}
			
			//draw map to screen
			Engine_BlitToRegion(kseMap[iMID].Map,
							 CInt(kseMap[iMID].iX) - 1,
							 CInt(kseMap[iMID].iY) - 1,
							 CInt(kseImg[kseMap[iMID].Map].imgWidth),
							 CInt(kseImg[kseMap[iMID].Map].imgHeight),
							 0,
							 quixieSCREEN);
			
			//check for additional process from user function
			Engine_RunMapScript(kseMap[iMID].sName + "_onEnterFrame", 0);
			
			Engine_Render();
			
			Engine_CheckForMovement();
			
			Engine_SpriteBillboardCheckup(); //this updates the rendering order of sprites
		} while (kseMap[iMID].bRendering == true); //while not being paused
		
		if (kseMap[iMID].bRender == true) {
			//check for pause function
			Engine_RunMapScript(kseMap[iMID].sName + "_onPause", 0);
		}
	} while (kseMap[iMID].bRender == true); //while not being ended
	
	Engine_RunMapScript(kseMap[iMID].sName + "_onUnload", 0);
	
	return iIndex;
}

////
// (<b>Dev API</b>) Goal is to give the illusion of Billboard concept (2D-in-3D) in DX8.
// This <i>updates</i> the "rendering <i>order</i>" of sprites.
//
// @remarks  None.
// @date 04.29.07
void Engine_SpriteBillboardCheckup() {
	int iMID = kseMAP_CURRENT_RENDER;
	for (int i = 1; i < kseMap[iMID].EntitiesCtr - 2; ++i) {
		for (int i2 = 2; i2 < kseMap[iMID].EntitiesCtr - 1; ++i2) {
			if (kseMap[iMID].Entities[i].iY > kseMap[iMID].Entities[i2].iY) {
				//switch the player ID if index is the assigned PLAYER
				if (i == kseMap[iMID].iPlayer) {
					kseMap[iMID].iPlayer = i2;
					kseMap[iMID].EntityFocus = i2;
				} else if (i2 == kseMap[iMID].iPlayer) {
					kseMap[iMID].iPlayer = i;
					kseMap[iMID].EntityFocus = i;
				}
				//place i to TMP
				
					kseMap[iMID].Entities[0].bHide = kseMap[iMID].Entities[i].bHide;
					kseMap[iMID].Entities[0].bStatic = kseMap[iMID].Entities[i].bStatic;
					kseMap[iMID].Entities[0].bUsed = kseMap[iMID].Entities[i].bUsed;
					
//					kseMap[iMID].Entities[0].iAnmt_DownTileMax = kseMap[iMID].Entities[i].iAnmt_DownTileMax;
//						fbUDTtoUDT kseMap[iMID].Entities[i].iAnmt_DownTiles, kseMap[iMID].Entities[0].iAnmt_DownTiles, kseMap[iMID].Entities[i].iAnmt_DownTileMax);
						std::vector<int>::iterator it;
						it = kseMap[iMID].Entities[i].iAnmt_DownTiles.begin();
						kseMap[iMID].Entities[0].iAnmt_DownTiles.assign(it, kseMap[iMID].Entities[i].iAnmt_DownTiles.end());
//					kseMap[iMID].Entities[0].iAnmt_LeftTileMax = kseMap[iMID].Entities[i].iAnmt_LeftTileMax;
//						fbUDTtoUDT .iAnmt_LeftTiles, kseMap[iMID].Entities[0].iAnmt_LeftTiles, kseMap[iMID].Entities[i].iAnmt_LeftTileMax);
						it = kseMap[iMID].Entities[i].iAnmt_LeftTiles.begin();
						kseMap[iMID].Entities[0].iAnmt_LeftTiles.assign(it, kseMap[iMID].Entities[i].iAnmt_LeftTiles.end());
//					kseMap[iMID].Entities[0].iAnmt_RightTileMax = kseMap[iMID].Entities[i].iAnmt_RightTileMax;
//						fbUDTtoUDT .iAnmt_RightTiles, kseMap[iMID].Entities[0].iAnmt_RightTiles, kseMap[iMID].Entities[i].iAnmt_RightTileMax);
						it = kseMap[iMID].Entities[i].iAnmt_RightTiles.begin();
						kseMap[iMID].Entities[0].iAnmt_RightTiles.assign(it, kseMap[iMID].Entities[i].iAnmt_RightTiles.end());
//					kseMap[iMID].Entities[0].iAnmt_UpTileMax = kseMap[iMID].Entities[i].iAnmt_UpTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[i].iAnmt_UpTiles, kseMap[iMID].Entities[0].iAnmt_UpTiles, kseMap[iMID].Entities[i].iAnmt_UpTileMax);
						it = kseMap[iMID].Entities[i].iAnmt_UpTiles.begin();
						kseMap[iMID].Entities[0].iAnmt_UpTiles.assign(it, kseMap[iMID].Entities[i].iAnmt_UpTiles.end());
					kseMap[iMID].Entities[0].iDirection = kseMap[iMID].Entities[i].iDirection;
					kseMap[iMID].Entities[0].iHeight = kseMap[iMID].Entities[i].iHeight;
					kseMap[iMID].Entities[0].IsAttachedToMap = kseMap[iMID].Entities[i].IsAttachedToMap;
//					kseMap[iMID].Entities[0].iSpriteHandlerMax = kseMap[iMID].Entities[i].iSpriteHandlerMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[i].iSpriteHandlers, kseMap[iMID].Entities[0].iSpriteHandlers, kseMap[iMID].Entities[i].iSpriteHandlerMax);
						it = kseMap[iMID].Entities[i].iSpriteHandlers.begin();
						kseMap[iMID].Entities[0].iSpriteHandlers.assign(it, kseMap[iMID].Entities[i].iSpriteHandlers.end());
//						kseMap[iMID].Entities[0].iSpriteIndex = kseMap[iMID].Entities[i].iSpriteIndex;
					kseMap[iMID].Entities[0].iWidth = kseMap[iMID].Entities[i].iWidth;
					kseMap[iMID].Entities[0].iX = kseMap[iMID].Entities[i].iX;
					kseMap[iMID].Entities[0].iY = kseMap[iMID].Entities[i].iY;
					kseMap[iMID].Entities[0].sName = kseMap[iMID].Entities[i].sName;
				
				//place i2 to i
				
					kseMap[iMID].Entities[i].bHide = kseMap[iMID].Entities[i2].bHide;
					kseMap[iMID].Entities[i].bStatic = kseMap[iMID].Entities[i2].bStatic;
					kseMap[iMID].Entities[i].bUsed = kseMap[iMID].Entities[i2].bUsed;
//					kseMap[iMID].Entities[i].iAnmt_DownTileMax = kseMap[iMID].Entities[i2].iAnmt_DownTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[i2].iAnmt_DownTiles, kseMap[iMID].Entities[i].iAnmt_DownTiles, kseMap[iMID].Entities[i2].iAnmt_DownTileMax);
						it = kseMap[iMID].Entities[i2].iAnmt_DownTiles.begin();
						kseMap[iMID].Entities[i].iAnmt_DownTiles.assign(it, kseMap[iMID].Entities[i2].iAnmt_DownTiles.end());
//					kseMap[iMID].Entities[i].iAnmt_LeftTileMax = kseMap[iMID].Entities[i2].iAnmt_LeftTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[i2].iAnmt_LeftTiles, kseMap[iMID].Entities[i].iAnmt_LeftTiles, kseMap[iMID].Entities[i2].iAnmt_LeftTileMax);
						it = kseMap[iMID].Entities[i2].iAnmt_LeftTiles.begin();
						kseMap[iMID].Entities[i].iAnmt_LeftTiles.assign(it, kseMap[iMID].Entities[i2].iAnmt_LeftTiles.end());
//					kseMap[iMID].Entities[i].iAnmt_RightTileMax = kseMap[iMID].Entities[i2].iAnmt_RightTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[i2].iAnmt_RightTiles, kseMap[iMID].Entities[i].iAnmt_RightTiles, kseMap[iMID].Entities[i2].iAnmt_RightTileMax);
						it = kseMap[iMID].Entities[i2].iAnmt_RightTiles.begin();
						kseMap[iMID].Entities[i].iAnmt_RightTiles.assign(it, kseMap[iMID].Entities[i2].iAnmt_RightTiles.end());
//					kseMap[iMID].Entities[i].iAnmt_UpTileMax = kseMap[iMID].Entities[i2].iAnmt_UpTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[i2].iAnmt_UpTiles, kseMap[iMID].Entities[i].iAnmt_UpTiles, kseMap[iMID].Entities[i2].iAnmt_UpTileMax);
						it = kseMap[iMID].Entities[i2].iAnmt_UpTiles.begin();
						kseMap[iMID].Entities[i].iAnmt_UpTiles.assign(it, kseMap[iMID].Entities[i2].iAnmt_UpTiles.end());
					kseMap[iMID].Entities[i].iDirection = kseMap[iMID].Entities[i2].iDirection;
					kseMap[iMID].Entities[i].iHeight = kseMap[iMID].Entities[i2].iHeight;
					kseMap[iMID].Entities[i].IsAttachedToMap = kseMap[iMID].Entities[i2].IsAttachedToMap;
//					kseMap[iMID].Entities[i].iSpriteHandlerMax = kseMap[iMID].Entities[i2].iSpriteHandlerMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[i2].iSpriteHandlers, kseMap[iMID].Entities[i].iSpriteHandlers, kseMap[iMID].Entities[i2].iSpriteHandlerMax);
						it = kseMap[iMID].Entities[i2].iSpriteHandlers.begin();
						kseMap[iMID].Entities[i].iSpriteHandlers.assign(it, kseMap[iMID].Entities[i2].iSpriteHandlers.end());
					kseMap[iMID].Entities[i].iSpriteIndex = kseMap[iMID].Entities[i2].iSpriteIndex;
					kseMap[iMID].Entities[i].iWidth = kseMap[iMID].Entities[i2].iWidth;
					kseMap[iMID].Entities[i].iX = kseMap[iMID].Entities[i2].iX;
					kseMap[iMID].Entities[i].iY = kseMap[iMID].Entities[i2].iY;
					kseMap[iMID].Entities[i].sName = kseMap[iMID].Entities[i2].sName;
				
				//place TMP to i2
				
					kseMap[iMID].Entities[i2].bHide = kseMap[iMID].Entities[0].bHide;
					kseMap[iMID].Entities[i2].bStatic = kseMap[iMID].Entities[0].bStatic;
					kseMap[iMID].Entities[i2].bUsed = kseMap[iMID].Entities[0].bUsed;
//					kseMap[iMID].Entities[i2].iAnmt_DownTileMax = kseMap[iMID].Entities[0].iAnmt_DownTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[0].iAnmt_DownTiles, kseMap[iMID].Entities[i2].iAnmt_DownTiles, kseMap[iMID].Entities[0].iAnmt_DownTileMax);
						it = kseMap[iMID].Entities[0].iAnmt_DownTiles.begin();
						kseMap[iMID].Entities[i2].iAnmt_DownTiles.assign(it, kseMap[iMID].Entities[0].iAnmt_DownTiles.end());
//					kseMap[iMID].Entities[i2].iAnmt_LeftTileMax = .iAnmt_LeftTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[0].iAnmt_LeftTiles, kseMap[iMID].Entities[i2].iAnmt_LeftTiles, kseMap[iMID].Entities[0].iAnmt_LeftTileMax);
						it = kseMap[iMID].Entities[0].iAnmt_LeftTiles.begin();
						kseMap[iMID].Entities[i2].iAnmt_LeftTiles.assign(it, kseMap[iMID].Entities[0].iAnmt_LeftTiles.end());
//					kseMap[iMID].Entities[i2].iAnmt_RightTileMax = .iAnmt_RightTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[0].iAnmt_RightTiles, kseMap[iMID].Entities[i2].iAnmt_RightTiles, kseMap[iMID].Entities[0].iAnmt_RightTileMax);
						it = kseMap[iMID].Entities[0].iAnmt_RightTiles.begin();
						kseMap[iMID].Entities[i2].iAnmt_RightTiles.assign(it, kseMap[iMID].Entities[0].iAnmt_RightTiles.end());
//					kseMap[iMID].Entities[i2].iAnmt_UpTileMax = .iAnmt_UpTileMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[0].iAnmt_UpTiles, kseMap[iMID].Entities[i2].iAnmt_UpTiles, kseMap[iMID].Entities[0].iAnmt_UpTileMax);
						it = kseMap[iMID].Entities[0].iAnmt_UpTiles.begin();
						kseMap[iMID].Entities[i2].iAnmt_UpTiles.assign(it, kseMap[iMID].Entities[0].iAnmt_UpTiles.end());
					kseMap[iMID].Entities[i2].iDirection = kseMap[iMID].Entities[0].iDirection;
					kseMap[iMID].Entities[i2].iHeight = kseMap[iMID].Entities[0].iHeight;
					kseMap[iMID].Entities[i2].IsAttachedToMap = kseMap[iMID].Entities[0].IsAttachedToMap;
//					kseMap[iMID].Entities[i2].iSpriteHandlerMax = kseMap[iMID].Entities[0].iSpriteHandlerMax;
//						fbUDTtoUDT(kseMap[iMID].Entities[0].iSpriteHandlers, kseMap[iMID].Entities[i2].iSpriteHandlers, kseMap[iMID].Entities[0].iSpriteHandlerMax);
						it = kseMap[iMID].Entities[0].iSpriteHandlers.begin();
						kseMap[iMID].Entities[i2].iSpriteHandlers.assign(it, kseMap[iMID].Entities[0].iSpriteHandlers.end());
					kseMap[iMID].Entities[i2].iSpriteIndex = kseMap[iMID].Entities[0].iSpriteIndex;
					kseMap[iMID].Entities[i2].iWidth = kseMap[iMID].Entities[0].iWidth;
					kseMap[iMID].Entities[i2].iX = kseMap[iMID].Entities[0].iX;
					kseMap[iMID].Entities[i2].iY = kseMap[iMID].Entities[0].iY;
					kseMap[iMID].Entities[i2].sName = kseMap[iMID].Entities[0].sName;
				
			}
		}
	}
}

////
// (<b>Dev API</b>) Checks if player pressed a key.
//
// @remarks This is the function that hooks the user input to the game.
void Engine_CheckForMovement() {
	//sentinel for checking if player was obstructed; look at the bottom part of this routine
	int oldX;
	int oldY;
	
	kseDIRECTION newDir;
	bool bMoveMap;
	int iMID;
	iMID = kseMAP_CURRENT_RENDER;
	
	
		kseMap[iMID].bMoveLeft = bMapMoveLeft;
		kseMap[iMID].bMoveRight = bMapMoveRight;
		kseMap[iMID].bMoveUp = bMapMoveUp;
		kseMap[iMID].bMoveDown = bMapMoveDown;
		kseMap[iMID].bRender = bMapRender;
		kseMap[iMID].bRendering = bMapRendering;
	
	
	//is it valid to check for new movement
	if (kseMap[iMID].iMoveMode == kseMAP_MOVE_TILEBASED_MODE
			&& iMapMoveCtr >= iMapMoveCtrMax) {
		//check for new movement
		if (kseMap[iMID].bMoveUp == true) {
			newDir = dirUP;
		} else if (kseMap[iMID].bMoveDown == true) {
			newDir = dDOWN;
		} else if (kseMap[iMID].bMoveLeft == true) {
			newDir = dLEFT;
		} else if (kseMap[iMID].bMoveRight == true) {
			newDir = dRIGHT;
		}
	} else {
		bMoveMap = true;
	}
	
	if (newDir != dNOT_SET) {
		//set new direction of //player/character//
		Engine_EntityChangeDirection(kseMap[iMID].iPlayer, newDir);
		//there should be no obstruction
		if (Engine_IsTileObstruct(kseMap[iMID].iPlayer, CInt(iMID)) == 0) {
			//get array count of a direction-animation
			//Print "got new direction:" + newDir
			iMapMoveCtr = 0;
			//Select Case newDir
			//	case dDOWN:
			//		iMapMoveCtrMax = kseEntity[kseEntity[iIdx].iSpriteIndex].iAnmt_DownTiles.size();
			//		break;
			//	case dirUP:
			//		iMapMoveCtrMax = kseEntity[kseEntity[iIdx].iSpriteIndex].iAnmt_UpTiles.size();
			//		break;
			//	case dLEFT:
			//		iMapMoveCtrMax = kseEntity[kseEntity[iIdx].iSpriteIndex].iAnmt_LeftTiles.size();
			//		break;
			//	case dRIGHT:
			//		iMapMoveCtrMax = kseEntity[kseEntity[iIdx].iSpriteIndex].iAnmt_RightTiles.size();
			//		break;
			//End Select
			iMapMoveCtrMax = kseMap[iMID].MapDimension;
		}
	}
	
	//set default value
	if (bMapFocusingOnEntity == true) {
		oldX = kseMap[iMID].iX;
		oldY = kseMap[iMID].iY;
	} else {
		oldX = kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX;
		oldY = kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY;
	}
	
	if (kseMap[iMID].iMoveMode == kseMAP_MOVE_TILEBASED_MODE
			&& iMapMoveCtr < iMapMoveCtrMax) { //bMoveMap
		//trace "iMapMoveCtr-" + iMapMoveCtr
		//TILE-BASED-MOVEMENT MODE
		//default and the only working one (for now)
		
		iMapMove = 1; //Math.Round(kseMap[imid].MapDimension / iMapMoveCtrMax)
		++iMapMoveCtr; //iMapMove
		int iMapDim = kseMap[iMID].MapDimension;


		if (bMapFocusingOnEntity == true) {
			
				oldX = kseMap[iMID].iX;
				oldY = kseMap[iMID].iY;
				
				switch (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iDirection) {
					case dDOWN:
						//limit moving of map upwards; entity is trying to move down
						if ((iMapDim - kseMap[iMID].Entities[kseMap[iMID].iPlayer].iHeight) < ((iMapDim * kseMap[iMID].iHeight) - (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY + kseMap[iMID].Entities[kseMap[iMID].iPlayer].iHeight))) {
							kseMap[iMID].iY = kseMap[iMID].iY - iMapMove;
							++kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY;
						}
						break;
					case dirUP:
						//limit moving of map downwards; entity is trying to move up
						if (kseMap[iMID].iY < ((kseImg[quixieSCREEN].imgHeight - kseMap[iMID].Entities[kseMap[iMID].iPlayer].iHeight) / 2)) {
							kseMap[iMID].iY = kseMap[iMID].iY + iMapMove;
							++kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY;
						}
						break;
					case dLEFT:
						//limit moving of map rightwards; entity is trying to move left
						if (kseMap[iMID].iX < ((kseImg[quixieSCREEN].imgWidth - kseMap[iMID].Entities[kseMap[iMID].iPlayer].iWidth) / 2)) {
							kseMap[iMID].iX = kseMap[iMID].iX + iMapMove;
							++kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX;
						}
						break;
					case dRIGHT:
						//limit moving of map leftwards; entity is trying to move right
						if ((iMapDim - kseMap[iMID].Entities[kseMap[iMID].iPlayer].iWidth) < ((iMapDim * kseMap[iMID].iWidth) - (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX + kseMap[iMID].Entities[kseMap[iMID].iPlayer].iWidth))) {
							kseMap[iMID].iX = kseMap[iMID].iX - iMapMove;
							++kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX;
						}
						break;
					case dNOT_SET:
						break;
					default:
						break;
				}
			

		} else {
//				oldX = kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX;
//				oldY = kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY;
				switch (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iDirection) {
					case dDOWN:
						//limit moving of map upwards; entity is trying to move down
						if (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY < (iMapDim * (kseMap[iMID].iHeight - 1))) {
						//if (kseMap[iMID].iY > XXX) {
							++kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY;
						}
						break;
					case dirUP:
						//limit moving of map downwards; entity is trying to move up
						if (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY > 0) {
							--kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY;
						}
						break;
					case dLEFT:
						//limit moving of map rightwards; entity is trying to move left
						if (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX > 0) {
							--kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX;
						}
						break;
					case dRIGHT:
						//limit moving of map leftwards; entity is trying to move right
						if ((iMapDim - kseMap[iMID].Entities[kseMap[iMID].iPlayer].iWidth)
							< 
							(
								(iMapDim * kseMap[iMID].iWidth) - 
								(kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX + kseMap[iMID].Entities[kseMap[iMID].iPlayer].iWidth)
							)) {
//						if (kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX < (iMapDim * (kseMap[iMID].iWidth - 1)) {
							kseMap[iMID].iX = kseMap[iMID].iX - iMapMove;
							++kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX;
						}
					case dNOT_SET:
						break;
					default:
						break;
				}
			
		}
	} else {
		//FREE MODE
		//TODO:study how RELSoft's scroll engine works
		//if (kseMap[iMID].bMoveUp == true) {
		//	kseMap[iMID].iY = kseMap[iMID].iY + iMapMove; //kseMAP_MOVE
		//} else if (kseMap[iMID].bMoveDown == true) {
		//	kseMap[iMID].iY = kseMap[iMID].iY - iMapMove; //kseMAP_MOVE
		//}
		//If kseMap[iMID].bMoveLeft == true) {
		//	kseMap[iMID].iX = kseMap[iMID].iX + iMapMove; //kseMAP_MOVE
		//} else if (kseMap[iMID].bMoveRight == true) {
		//	kseMap[iMID].iX = kseMap[iMID].iX - iMapMove; //kseMAP_MOVE
		//}
	}

	//animate //player/character// only when moving
	if (bMapFocusingOnEntity == true) {
		if (oldX == kseMap[iMID].iX && oldY == kseMap[iMID].iY) {
			kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic = true;
		} else {
			kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic = false;
		}
	} else {
		if (oldX == kseMap[iMID].Entities[kseMap[iMID].iPlayer].iX
				&& oldY == kseMap[iMID].Entities[kseMap[iMID].iPlayer].iY) {
			kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic = true;
		} else {
			kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic = false;
		}
	}
}

////
// (<b>Dev API</b>) Executes a function related to map.
//
// @param	fxName - name of function to execute.
// @param	bWarn - ID of map to be checked.
// @remarks Function wrapper for map files. -- it uses <b>Engine_RunScript</b>.
void Engine_RunMapScript(string fxName, bool bWarn) {
	Engine_RunScript(sMapName, fxName, "", bWarn);
}


////
// (<b>Dev API</b>) Returns a sprite-image-buffer of an entity to be displayed in <b>Game:Render</b>.
//
// @param	iEntity - Which entity among the attached entity is to be checked?
// @return   Frame index of the sprite-animation to be rendered.
// @remarks None.
int KSE_GetSpriteAnimation(int iEntity) {
	unsigned int iEntCtr;
	int iSprIdx;
	int iFmeCtr;  //animate or not
	kseDIRECTION iDir;
	int iMID = kseMAP_CURRENT_RENDER;
	
	
		iDir = kseMap[iMID].Entities[iEntity].iDirection;
		if (kseMap[iMID].Entities[iEntity].bStatic == false) { iFmeCtr = 1; }
		
		if (iDir == dDOWN) {
			//get current counter then add one
			iEntCtr = kseMap[iMID].Entities[iEntity].iAnmt_DownCtr + iFmeCtr;
			
			//check for out-of-bounds
			if (iEntCtr >= kseMap[iMID].Entities[iEntity].iAnmt_DownTiles.size()) { iEntCtr = 0; }
			
			//set new counter value
			kseMap[iMID].Entities[iEntity].iAnmt_DownCtr = iEntCtr;
			
			//get tile based from counter value
			iSprIdx = kseMap[iMID].Entities[iEntity].iAnmt_DownTiles[iEntCtr];
		} else if (iDir == dLEFT) {
			//get current counter then add one
			iEntCtr = kseMap[iMID].Entities[iEntity].iAnmt_LeftCtr + iFmeCtr;
			
			//check for out-of-bounds
			if (iEntCtr >= kseMap[iMID].Entities[iEntity].iAnmt_LeftTiles.size()) { iEntCtr = 0; }
			
			//set new counter value
			kseMap[iMID].Entities[iEntity].iAnmt_LeftCtr = iEntCtr;
			
			//get tile based from counter value
			iSprIdx = kseMap[iMID].Entities[iEntity].iAnmt_LeftTiles[iEntCtr];
		} else if (iDir == dRIGHT) {
			//get current counter then add one
			iEntCtr = kseMap[iMID].Entities[iEntity].iAnmt_RightCtr + iFmeCtr;
			
			//check for out-of-bounds
			if (iEntCtr >= kseMap[iMID].Entities[iEntity].iAnmt_RightTiles.size()) { iEntCtr = 0; }
			
			//set new counter value
			kseMap[iMID].Entities[iEntity].iAnmt_RightCtr = iEntCtr;
			
			//get tile based from counter value
			iSprIdx = kseMap[iMID].Entities[iEntity].iAnmt_RightTiles[iEntCtr];
		} else if (iDir == dirUP) {
			//get current counter then add one
			iEntCtr = kseMap[iMID].Entities[iEntity].iAnmt_UpCtr + iFmeCtr;
			
			//check for out-of-bounds
			if (iEntCtr >= kseMap[iMID].Entities[iEntity].iAnmt_UpTiles.size()) { iEntCtr = 0; }
			
			//set new counter value
			kseMap[iMID].Entities[iEntity].iAnmt_UpCtr = iEntCtr;
			
			//get tile based from counter value
			iSprIdx = kseMap[iMID].Entities[iEntity].iAnmt_UpTiles[iEntCtr];
		}
		
		return kseMap[iMID].Entities[iEntity].iSpriteHandlers[iSprIdx];
	
}

////
// <b>Game:FocusEntity</b> - Gives focus on an entity.
//
// @param	iIndex - Index of the parsed script.
// @return   New index value of the parsed script.
// @remarks  None.
//<div class="KS">
// <b>KonsolScript Users:</b><br>
// <br><b>Description:</b><br>
// Gives game camera's focus on a specific entity.
// <br><br><b>Parameters:</b><br>
// <ul>
//  <li>string <b>entity</b> - instance name of the entity to be focused.
// </ul>
// <b>Usage:</b><br>
// <pre>
//  //provided testMap is a defined map (see Map:New)
//  Game:Render(testMap)
//  //provided that myEntity is an instance of a defined entity (See Game:AttachEntity)
//  Game:FocusEntity("myEntity")
// </pre>
//</div>
unsigned long GameFocusEntity(unsigned long &iIndex) {
	string sEntity;
	
	iIndex = Game_SkipOpenParenthesisAfter("FocusEntity", iIndex);
		
		sEntity = GetStrVal(iIndex, 1, 0);
		
	iIndex = Game_SkipCloseParenthesisAfter("entity", iIndex);
	//EXECUTE SCRIPT =====================================================================
	bool bGoCheck;
	int iMID = kseMAP_CURRENT_RENDER;
	if (kseMap[iMID].iMoveMode == kseMAP_MOVE_TILEBASED_MODE) {
		//make sure to finish player's tile-based movement
		if (iMapMoveCtr >= iMapMoveCtrMax) {
			bGoCheck = true;
		}
	} else {
		bGoCheck = true;
	}
	int i;
	if (bGoCheck == true) {
		if (sEntity != "") {
			bMapFocusingOnEntity = true;
			
			for (i = 1; i < kseMap[iMID].EntitiesCtr - 1; ++i) {
				if (LCase(kseMap[iMID].Entities[i].sName) == LCase(sEntity)) {
					if (kseMap[iMID].EntityFocus != i) {
						//NOTE: this condition should be kept separated with the first 2 condition
						if ((kseMap[iMID].iPlayer != 0 && kseMap[iMID].Entities[kseMap[iMID].iPlayer].bStatic == true)
								|| kseMap[iMID].iPlayer == 0) {
							//change current player only when current player-entity is not moving
							kseMap[iMID].EntityFocus = i;
							kseMap[iMID].iX = (-kseMap[iMID].Entities[i].iX) + (kseImg[quixieSCREEN].imgWidth / 2) - (kseMap[iMID].Entities[i].iWidth / 2);
							kseMap[iMID].iY = (-kseMap[iMID].Entities[i].iY) + (kseImg[quixieSCREEN].imgHeight / 2) - (kseMap[iMID].Entities[i].iHeight / 2);
						}
					}
					break;
				}
			}
		} else {
			//remove focus to entity
			if (kseMap[iMID].EntityFocus > 0) {
				i = kseMap[iMID].EntityFocus;
				kseMap[iMID].iX = (-kseMap[iMID].Entities[i].iX) + (kseImg[quixieSCREEN].imgWidth / 2) - (kseMap[iMID].Entities[i].iWidth / 2);
				kseMap[iMID].iY = (-kseMap[iMID].Entities[i].iY) + (kseImg[quixieSCREEN].imgHeight / 2) - (kseMap[iMID].Entities[i].iHeight / 2);
			}
			kseMap[iMID].EntityFocus = 0;
			
			kseMap[iMID].iCamX = -kseMap[iMID].iX;
			kseMap[iMID].iCamY = -kseMap[iMID].iY;
			
			bMapFocusingOnEntity = 0;
		}
	}
	
	return iIndex;
}

////
// (<b>Dev API</b>) Returns a tile-image-buffer to be displayed in <b>Game:Render</b>.
//
// @param	iTileIdx - Index of the tile to be checked
// @param	iMID - ID of map to be checked.
// @return   See remarks for better understanding; this will return the <b>z</b>.
// @remarks Guide to this algo <i>(it's driving me crazy!)</i><br>
//DELAY is PROGRAMMED as <b>x / y = z</b> <br>
//<b>x</b> - blitting counter; if it reaches the value of y, add 1 to z <br>
//<b>y</b> - number of times for a single frame to be blitted <br>
//<b>z</b> - current frame being shown <br>
int KSE_GetTileAnimation(int iTileIdx, int iMID) {
	unsigned int iTileCtr;	  //our z
	int iTileCtr2;  //our x
	int iTileIdx2;
	//get index of "tile-animation array"
	iTileIdx2 = kseMap[iMID].iTileData[iTileIdx].iIdx;
	//get index of the current frame from tile-animation to be shown
	iTileCtr2 = kseMap[iMID].iTileData[iTileIdx].iCtr + 1;
	
	//get old counter then add one
	iTileCtr = kseTileAnimation[iTileIdx2].iCtr;
	
	if (iTileCtr2 > kseMap[iMID].iTileData[iTileIdx].iDelay) {
		iTileCtr2 = 0;
		//add one to current counter
		++iTileCtr;
	}
	
	//check for out-of-bounds
	if (iTileCtr > kseTileAnimation[iTileIdx2].iTiles.size()) { iTileCtr = 0; }
	//set new counter value
	kseTileAnimation[iTileIdx2].iCtr = iTileCtr;
	
	kseMap[iMID].iTileData[iTileIdx].iCtr = iTileCtr2;
	
	//get tile based from counter value
	int iTileIndex = kseTileAnimation[iTileIdx2].iTiles[iTileCtr];
	if (iTileIndex < 0 ) {
		iTileIndex = kseTileAnimation[iTileIdx2].iTiles[1]; //[QuixiePP] why not give slot zero?
	}
	
	--iTileIndex;
		if (iTileIndex < 0 ) { iTileIndex = 0; }
	
	return iTileIndex;
}

////
// (<b>Dev API</b>) Game-class// wrapper for skipping open parenthesis.
//
// @param	sWhat - an open parenthesis should be written next to what text?
// @param	iIndex - Index of the parsed script
// @return   New index value of the parsed script.
// @remarks  Benchmark if organizing is worth the speed.
unsigned long Game_SkipOpenParenthesisAfter(string sWhat, unsigned long &iIndex) {
	cGameTxt = "Game:" + sWhat;
	return KSE_SkipOpenParenthesisAfter(cGameTxt, iIndex);
}

////
// (<b>Dev API</b>) Game-class// wrapper for skipping a comma.
//
// @param	sWhat - a comma should be written next to what text?
// @param	iIndex - Index of the parsed script
// @return   New index value of the parsed script.
// @remarks  Benchmark if organizing is worth the speed.
unsigned long Game_SkipCommaAfter(string sWhat, unsigned long &iIndex) {
	cGameTxt = cGameTxt + sWhat;
		iIndex = KSE_SkipCommaAfter(cGameTxt, iIndex);
	cGameTxt = cGameTxt + ", ";
	
	return iIndex;
}

////
// (<b>Dev API</b>) Game-class// wrapper for skipping close parenthesis.
//
// @param	sWhat - a close parenthesis should be written next to what text?
// @param	iIndex - Index of the parsed script
// @return   New index value of the parsed script.
// @remarks  Benchmark if organizing is worth the speed.
unsigned long Game_SkipCloseParenthesisAfter(string sWhat, unsigned long &iIndex) {
	cGameTxt = cGameTxt + sWhat;
		iIndex = KSE_SkipCloseParenthesisAfter(cGameTxt, iIndex);
	cGameTxt = "";
	return iIndex;
}
